/*===========================================================================

	Copyright  2007 Tea Tuar

	This material is provided "as is", with no warranty expressed or implied.
	Any use is at your own risk. Permission to use or copy this software for
	any purpose is hereby granted without fee, provided this notice is
	retained on all copies. Permission to modify the code and to distribute
	modified code is granted, provided a notice that the code was modified is
	included with the above copyright notice.

	http://dis.ijs.si/tea/research.htm

	===========================================================================

	Implementation file.

	file: demo_alg.cpp
	author: Tea Tusar (tea.tusar@ijs.si)
	created: 17.02.2006
	last change: $date$

	===========================================================================
*/

#include "demo_alg.h"
#include "demowsa_localsearch.h"
#include <algorithm>
//---------------------------------------------------------------------------

const UINT g_sel_NSGA = 0;
const UINT g_sel_IBEA = 1;
const UINT g_sel_SPEA = 2;
//---------------------------------------------------------------------------

const UINT g_trunc_k = 0;
const UINT g_trunc_1 = 1;
const UINT g_trunc_p_c = 2;
//---------------------------------------------------------------------------

DemoAlg::DemoAlg(string ini_f_name, string ini_args)
{
	timingsOnly=false;
	ifstream f_ini;
	f_ini.open(ini_f_name.c_str(), fstream::in);

	try {
		if (!f_ini.good()) {
			throw E("Could not open the file " + ini_f_name);
		}
	} catch (E &e) {
		cerr << e.error << endl;
		set_terminate(my_terminate);
		terminate();
	}

	m_params = new DemoParams();
	m_files = new FileStat();

	m_probl_fun = new ProblFunCec09();
	m_probl_sim = new ProblSim();

	m_pop_nsga = new DemoPopNsga();
	m_pop_ibea = new DemoPopIbea();
	m_pop_spea = new DemoPopSpea();

	m_seed = 0;
	m_sel_type = g_sel_NSGA;
	m_always_add = false;

	ReadFromFile(ini_f_name);
	if (ini_args != "") {
		ReadFromString(ini_args);
	}

	InitAlg();

	f_ini.close();
	m_DemowsaLocalsearch = new DemowsaLocalsearch();
}
//---------------------------------------------------------------------------

DemoAlg::~DemoAlg()
{
	delete m_params;
	delete m_files;
	delete m_probl_fun;
	delete m_probl_sim;
	delete m_pop_nsga;
	delete m_pop_ibea;
	delete m_pop_spea;
	delete m_DemowsaLocalsearch;
}
//---------------------------------------------------------------------------

void DemoAlg::ReadFromFile(string f_name)
{
	ifstream f;
	f.open(f_name.c_str(), fstream::in);

	ReadFromStream(f);
	f.close();
}
//---------------------------------------------------------------------------

void DemoAlg::ReadFromString(string args)
{
	istringstream str(args.c_str(), stringstream::in);

	ReadFromStream(str);
}
//---------------------------------------------------------------------------

void DemoAlg::ReadFromStream(istream &stream)
{
	string tmp_str;
	UINT tmp_int;
	double tmp_double;
	bool tmp_bool;

	try {
		while (!stream.eof()) {
			stream >> tmp_str;
			if (stream.eof())
				break;

			if (tmp_str == "%") {
				stream.ignore(256, '\n');
			}

			else if (tmp_str == "seed") {
				stream >> m_seed;
			}
			else if (tmp_str == "pop_size") {
				stream >> tmp_int;
				if (tmp_int < 5)
					throw E("Error in parameter \"pop_size\" (it should be greater than 4)");
				m_params->approximationSetSize=tmp_int;
				m_params->SetPopSize(m_params->NP_size_factor*tmp_int);
			}
			else if (tmp_str == "max_eval") {
				stream >> tmp_int;
				if (tmp_int < 5)
					throw E("Error in parameter \"max_eval\" (it should be greater than 4)");
				m_params->SetMaxEval(tmp_int);
			}
			else if (tmp_str == "weight") {
				stream >> tmp_double;
				if ((tmp_double < 0) || (tmp_double > 2))
					throw E("Error in parameter \"weight\" (it should lie in interval [0, 2])");
				m_params->SetWeight(tmp_double);
			}
			else if (tmp_str == "cross_prob") {
				stream >> tmp_double;
				if ((tmp_double < 0) || (tmp_double > 1))
					throw E("Error in parameter \"cross_prob\" (it should lie in interval [0, 1])");
				m_params->SetCrossProb(tmp_double);
			}
			else if (tmp_str == "sel_type") {
				stream >> m_sel_type;
				if ((m_sel_type != g_sel_NSGA) && (m_sel_type != g_sel_IBEA)
					&& (m_sel_type != g_sel_SPEA))
					throw E("Error in parameter \"sel_type\"");
			}
			else if (tmp_str == "always_add") {
				stream >> m_always_add;
			}

			else if (tmp_str == "elitism") {
				stream >> tmp_bool;
				m_pop_nsga->SetElitism(tmp_bool);
			}
			else if (tmp_str == "elitism_r") {
				stream >> tmp_double;
				if ((tmp_double <= 0) || (tmp_double > 1))
					throw E("Error in parameter \"elitism_r\" (it should lie in interval (0, 1]");
				m_pop_nsga->SetElitismR(tmp_double);
			}
			else if (tmp_str == "indicator") {
				stream >> tmp_int;
				if ((tmp_int != g_ibea_eps) && (tmp_int != g_ibea_hyp))
					throw E("Error in parameter \"indicator\"");
				m_pop_ibea->SetIndicator(tmp_int);
			}
			else if (tmp_str == "rho") {
				stream >> tmp_double;
				if (tmp_double <= 1)
					throw E("Error in parameter \"rho\" (it should be greater than 1)");
				m_pop_ibea->SetRho(tmp_double);
			}
			else if (tmp_str == "kappa") {
				stream >> tmp_double;
				if ((tmp_double <= 0) || (tmp_double > 1))
					throw E("Error in parameter \"kappa\" (it should lie in interval (0, 1])");
				m_pop_ibea->SetKappa(tmp_double);
			}

			else if (tmp_str == "func_type") {
				stream >> tmp_int;
				if ((tmp_int < g_prob_min) || (tmp_int > g_prob_max))
					throw E("Error in parameter \"func_type\"");
				m_probl_sim->SetFunction(tmp_int);
				m_probl_fun->SetFunction(tmp_int);
			}
			else if (tmp_str == "num_var") {
				stream >> tmp_int;
				if (tmp_int < 1)
					throw E("Error in parameter \"num_var\" (it should be greater than 0)");
				m_probl_sim->SetNumVar(tmp_int);

				vector<double> var_min;
				vector<double> var_max;
				vector<double> var_disc;

				for (UINT i = 0; i < tmp_int; i++) {
					stream >> tmp_double; var_min.push_back(tmp_double);
					stream >> tmp_double; var_max.push_back(tmp_double);
					stream >> tmp_double; var_disc.push_back(tmp_double);
					if ((var_min.at(i) >= var_max.at(i)) ||
						(var_disc.at(i) > var_max.at(i) - var_min.at(i))) {
						throw E("Error in variable min/max/discretization");
					}
				}
				m_probl_sim->SetVarMin(var_min);
				m_probl_sim->SetVarMax(var_max);
				m_probl_sim->SetVarDisc(var_disc);
			}
			else if (tmp_str == "num_func") {
				stream >> tmp_int;
				if (tmp_int < 2)
					throw E("Error in parameter \"num_func\" (it should be greater than 1)");
				m_probl_sim->SetNumFunc(tmp_int);

				vector<double> func_lower;
				vector<double> func_upper;

				for (UINT i = 0; i < tmp_int; i++) {
					stream >> tmp_double; func_lower.push_back(tmp_double);
					stream >> tmp_double; func_upper.push_back(tmp_double);
					if (func_lower.at(i) >= func_upper.at(i)) {
						throw E("Error in function bounds");
					}
				}
				m_probl_sim->SetFuncLower(func_lower);
				m_probl_sim->SetFuncUpper(func_upper);
			}
			else if (tmp_str == "num_feat") {
				stream >> tmp_int;
				m_probl_sim->SetNumFeat(tmp_int);
			}

			else if (tmp_str == "command") {
				stream >> tmp_str;
				m_probl_sim->SetCommand(tmp_str);
			}
			else if (tmp_str == "archive") {
				stream >> tmp_str;
				m_probl_sim->SetArchive(tmp_str);
			}
			else if (tmp_str == "archive_read") {
				stream >> tmp_bool;
				m_probl_sim->SetArchiveRead(tmp_bool);
			}
			else if (tmp_str == "archive_write") {
				stream >> tmp_bool;
				m_probl_sim->SetArchiveWrite(tmp_bool);
			}
			else if (tmp_str == "sim_in") {
				stream >> tmp_str;
				m_probl_sim->SetSimIn(tmp_str);
			}
			else if (tmp_str == "sim_out") {
				stream >> tmp_str;
				m_probl_sim->SetSimOut(tmp_str);
			}

			else if (tmp_str == "wfg_M") {
				stream >> tmp_int;
				if (tmp_int < 2)
					throw E("Error in parameter \"wfg_M\" (it should be greater than 1)");
				m_probl_fun->SetWfgM(tmp_int);
			}
			else if (tmp_str == "wfg_k") {
				stream >> tmp_int;
				if (tmp_int < 1)
					throw E("Error in parameter \"wfg_k\" (it should be greater than 0)");
				if (tmp_int % (m_probl_fun->GetWfgM() - 1) != 0)
					throw E("Error in parameter \"wfg_k\" (wfg_M - 1 should divide wfg_k)");
				m_probl_fun->SetWfgK(tmp_int);
			}
			else if (tmp_str == "wfg_l") {
				stream >> tmp_int;
				if (tmp_int < 1)
					throw E("Error in parameter \"wfg_l\" (it should be greater than 0)");
				m_probl_fun->SetWfgL(tmp_int);
			}

			else if (tmp_str == "gen_file_name") {
				stream >> tmp_str;
				m_files->GetFileGen()->SetName(tmp_str);
			}
			else if (tmp_str == "gen_mode") {
				stream >> tmp_int;
				if ((tmp_int != g_mode_off) && (tmp_int != g_mode_overwrite)
					&& (tmp_int != g_mode_append))
					throw E("Error in parameter \"gen_mode\"");
				m_files->GetFileGen()->SetMode(tmp_int);
			}
			else if (tmp_str == "front_file_name") {
				stream >> tmp_str;
				m_files->GetFileFront()->SetName(tmp_str);
			}
			else if (tmp_str == "front_mode") {
				stream >> tmp_int;
				if ((tmp_int != g_mode_off) && (tmp_int != g_mode_overwrite)
					&& (tmp_int != g_mode_append))
					throw E("Error in parameter \"front_mode\"");
				m_files->GetFileFront()->SetMode(tmp_int);
			}
			else if (tmp_str == "front_gen") {
				stream >> tmp_int;
				if (tmp_int < 1)
					throw E("Error in parameter \"front_gen\" (it should be greater than 0)");
				m_files->SetFrontGen(tmp_int);
			}
			else if (tmp_str == "front_long") {
				stream >> tmp_bool;
				m_files->SetFrontLong(tmp_bool);
			}
			else if (tmp_str == "log_file_name") {
				stream >> tmp_str;
				m_files->GetFileLog()->SetName(tmp_str);
			}
			else if (tmp_str == "log_mode") {
				stream >> tmp_int;
				if ((tmp_int != g_mode_off) && (tmp_int != g_mode_overwrite)
					&& (tmp_int != g_mode_append))
					throw E("Error in parameter \"log_mode\"");
				m_files->GetFileLog()->SetMode(tmp_int);
			}
			else if (tmp_str == "lines_file_name") {
				stream >> tmp_str;
				m_files->GetFileLines()->SetName(tmp_str);
			}
			else if (tmp_str == "lines_mode") {
				stream >> tmp_int;
				if ((tmp_int != g_mode_off) && (tmp_int != g_mode_overwrite)
					&& (tmp_int != g_mode_append))
					throw E("Error in parameter \"lines_mode\"");
				m_files->GetFileLines()->SetMode(tmp_int);
			}
			else if (tmp_str == "strategyDE") {
				stream >> tmp_int;
				if ((tmp_int <= 0) && (tmp_int >= 200))
					throw E("Error in parameter \"strategyDE\"");
				m_params->strategyDE=tmp_int;
			}
			else if (tmp_str == "strategyPrunning") {
				stream >> tmp_int;
				if ((tmp_int <= 0) && (tmp_int >= 200))
					throw E("Error in parameter \"strategyPrunning\"");
				m_params->strategyPrunning=tmp_int;
			}
			else if (tmp_str == "timingsOnly") {
				stream >> tmp_int;
				if ((tmp_int <= 0) && (tmp_int >= 3))
					throw E("Error in parameter \"strategyDE\"");
				//0=no, 1=only functions, 2=algorithm runs too
				timingsOnly=tmp_int;
			}
			else {
				throw E("Unknown parameter: " + tmp_str);
			}
		}
	} catch (E &e) {
		cerr << e.error << endl;
		set_terminate(my_terminate);
		terminate();
	} catch (...) {
		cerr << "Error occurred when processing parameters!" << endl;
		set_terminate(my_terminate);
		terminate();
	}
}
//---------------------------------------------------------------------------

void DemoAlg::InitAlg()
{
	if (m_seed == 0) {
		//randomize();                 //patch by AlesZ
		m_seed = 3;/*random(MAXINT);*/ //patch by AlesZ
		std::cout << "m_seed initialized to 3!" << std::endl;
	}
	srand(m_seed);

	if (m_probl_fun->GetFunction() == g_prob_extern)
		m_probl = m_probl_sim;
	else
		m_probl = m_probl_fun;

	m_probl->InitProblem();

	m_files->InitFileStat();

	if (m_sel_type == g_sel_NSGA)
		m_pop = m_pop_nsga;
	else if (m_sel_type == g_sel_IBEA)
		m_pop = m_pop_ibea;
	else if (m_sel_type == g_sel_SPEA)
		m_pop = m_pop_spea;

	m_pop->InitPop(m_params, m_probl);

	m_pop_ibea->InitPop(m_params, m_probl);//for strategyPrunning==3


	//KukkonenCec07: max of 200 last generations
	m_appSet.resize(m_params->GetPopSize()*m_params->appsetMaxGenerations,0);
	m_appSet_currInd=0;

	m_files->m_params=m_params;


	OutputInfoGen();
	OutputInfoLog();
}
//---------------------------------------------------------------------------
void DemoAlg::checkCandidate(DemoInd &candidate) {
	for (int i = 0; i < m_probl->GetNumVar(); i++) {
		double g = candidate.GetGene(i);
		double L = m_probl->GetVarLower(i);
		double U = m_probl->GetVarUpper(i);
		if (g < L || g > U) {
			cout << "g=" << g << ", L=" << L << ", U="<< U<<endl;
			cout << "EEE" << endl;
		}
	}

	for (int i = 0; i < m_probl->GetNumFunc(); i++) {
		double c = candidate.GetCriterion(i);
		double L = m_probl->GetFuncLower(i);
		double U = m_probl->GetFuncUpper(i);
		if (c < L /*|| c > U*/) {
			cout << "c=" << c << ", L=" << L << ", U="<< U<<endl;
			cout << "EEE" << endl;
		}
	}
}

void DemoAlg::Evolution()
{
	if (timingsOnly==1) {
		TimingsCec07();
		return;
	}

	m_pop->CreateFirstPop();
	m_pop->EvaluatePop(m_files);
	initEpsilonConstraint();
//	m_files->OutputPop(m_pop, 1);
	vector<DemoInd*>::iterator cii;
	int i=0;
	for(cii=m_appSet.begin(); cii!=m_appSet.end(); cii++)
		*cii = new DemoInd(*m_pop->GetInd((i++)%m_params->GetPopSize()));

	///HAcK: disabled because of special SP4c
	///m_files->OutputPop(m_pop, 1, m_appSet);///here must be no print-out!


	vector<UINT> indexes;
	UINT domination;

	//Added by AlesZ <ales.zamuda@uni-mb.si>
	//Running (instantly updating, changing) approximate number of
	//individuals in the starting part of the population:
	//in the beginning, there will be no dominance measure,
	//but in the progress of generations, more and more
	//individuals in with lower indices in the population are
	//Pareto efficient
// 	int running_ApproxNrOfNondominatedInds=0;
// 	for (UINT i = 0; i < m_params->GetPopSize(); i++) {
// 		domination = m_probl->GetDomination(candidate, m_pop->GetInd(i));
// 	}
	bool firstGeneration=false;

	//calculate avgF and avgCR for init. generation
	double oldF=-1000,oldCR=-1000;
	for (UINT i = 0; i < m_params->GetPopSize(); i++) {
		double F=m_pop->GetInd(i)->GetWeightF(),CR=m_pop->GetInd(i)->GetCrossProbCR();
		oldF=(i*oldF + F)/(i+1);
		oldCR=(i*oldCR + CR)/(i+1);
	}
	int genNr=0;
        cout <<endl<< "Favg=" << oldF << " \tCRavg=" << oldCR<< " \tG=" << genNr << " \tFES="<<m_probl->GetNumEval();

	while (m_probl->GetNumEval() < m_params->GetMaxEval()) {
		genNr++;
		indexes = m_pop->SortPopRandom();
		m_pop->TruncPop(indexes);//time consuming: make a deep rearanged copy of individuals
		vector<int> nondominatedInd;//Nondominated indices of individuals
		generateNDList(nondominatedInd);

		if (m_params->strategyPrunning==1 || firstGeneration || m_params->strategyPrunning==4) {
			double avgF=-1000, avgCR=-1000; int nrBetter=0;//-1=never better
			for (UINT i = 0; i < m_params->GetPopSize(); i++) {
				DemoInd *candidate = new DemoInd();
				double F=m_params->GetWeight(),CR=m_params->GetCrossProb();//patch by AlesZ
				candidate->SetGenes(m_pop->GetCandidate(i, F,CR,candidate,nondominatedInd));//patch by AlesZ
				candidate->SetWeightF(F);//if strategy != 13, not importaint
				candidate->SetCrossProbCR(CR);//if strategy != 13, not importaint
				candidate->Evaluate(m_probl);

				domination = m_probl->GetDomination(candidate, m_pop->GetInd(i));
				candidate->SetDomination( domination);
				m_files->OutputInd(candidate,
													m_pop->GetInd(i),
													m_probl->GetNumEval(),
													domination);

				if (m_always_add) {
					m_pop->AppendInd(candidate);
				}
				else {
					if (domination == g_domin_first) {
						*m_pop->GetInd(i) = *candidate;
						delete candidate;
						avgF=(nrBetter*avgF + F)/(nrBetter+1);
						avgCR=(nrBetter*avgCR + CR)/(nrBetter+1);
						nrBetter++;
        					cout <<endl<< "Fbetter=" << F << " \tCRbetter=" << CR<< " \tG=" << genNr << " \tFES="<<m_probl->GetNumEval();
					}
					else if (domination == g_domin_second) {
						delete candidate;
					}
					else if (domination == g_domin_none) {
						m_pop->AppendInd(candidate);
					}
				}
//        			cout <<endl<< "Fall=" << F << " \tCRall=" << CR<<endl;
			}
			firstGeneration=false;

			indexes = m_pop->SortPop();
			m_pop->TruncPop(indexes);
			if (m_params->strategyPrunning==4) savePopulationToLongApproximationSet();
			if (avgF==-1000) {//remember the last BETTER F,CR
				avgF=oldF; avgCR=oldCR;
			} else {
				oldF=avgF; oldCR=avgCR;
			}
        		cout <<endl<< "Favg=" << avgF << " \tCRavg=" << avgCR<< " \tG=" << genNr << " \tFES="<<m_probl->GetNumEval();

			//tukaj LS naprej...
			float ratioFES_LS=0.1;
			static int currentFES_LS=0;

/*			int doLSeveryNthGen=300;//LS uses
			switch(m_probl->GetNumFunc()) {
				case 2:doLSeveryNthGen=500;break;//G=500,1000,1500 (100.000, 200.000, 300.000 FES)
				case 3:doLSeveryNthGen=333;break;//G=333,666,999  (99900, 199800, 299700 FES)
				case 5:doLSeveryNthGen=62;break;//G=62,124,186 (99200, 198400, 297600 FES)
			}
*/
			//actual:
			// OKA2: 300*20 -> 6.000 FES
			// M=3: 3500*20 -> 35.000 FES
			// DTLZ M=5: 90*20 -> 1.800 FES
			// WFG M=5: 150*176 -> 26.000 FES

//			static int nrLocalSearches=0;
			//do LS at 100.000, 200.000 (max. 250.000), 300.000 (max. 400.000) FES
			if (/*genNr%doLSeveryNthGen ==0 && nrLocalSearches<3*/
				/*currentFES_LS / m_probl->GetNumEval() + 50000 < ratioFES_LS && */genNr%25==0
				) {
//				nrLocaSearches++;
				int size = (int)m_pop->GetInds().size();
				int usedFESthisLS=0;
				int nrChecked=0;
				for (UINT i = 0; i < size; i++) {

//					if ((double) rand() / RAND_MAX < 0.9) continue;

					DemoInd *candidate=m_pop->GetInd(i);
					if (candidate->GetDomination()==g_domin_none) continue;//LS on only of EA-improved
					nrChecked++;
					if ((double) rand() / RAND_MAX > 0.10) continue;

//					if (nrChecked > int(0.15*size)) break;
					candidate->SetDomination( g_domin_none );//set to non-improvement

					double oldCmin0=candidate->GetCriteria()[0];
					double oldCmin1=candidate->GetCriteria()[1];
					int nfE=m_probl->GetNumEval();

//					checkCandidate(*candidate);
					DemoInd candidateLS = *candidate;
//					checkCandidate(candidateLS);
					//int miter=50; //m_params->GetMaxEval()-m_probl->GetNumEval()
					m_DemowsaLocalsearch->improve(candidateLS, *m_probl, 3);
					if (m_probl->GetDomination(&candidateLS, candidate)==g_domin_first) {
						*candidate=candidateLS;
                                                candidate->SetDomination( g_domin_first );//remember the SQP improvement

						m_DemowsaLocalsearch->improve(candidateLS, *m_probl, 10);
						if (m_probl->GetDomination(&candidateLS, candidate)==g_domin_first) {
							*candidate=candidateLS;
							candidate->SetDomination( g_domin_first );//remember the SQP improvement
						}
					}
//					checkCandidate(candidateLS);

					double cmin0=candidate->GetCriteria()[0];
					double cmin1=candidate->GetCriteria()[1];
					int nfeval=m_probl->GetNumEval();
					usedFESthisLS+=nfeval-nfE;

					std::cout << "\nFIN at gen" << genNr
						<< ", used FES= " << nfeval-nfE << " for candidate " << i << " , "
						<< "total this LS= " << usedFESthisLS << " @ FES " << nfeval
						<< " , cminDiff(must be positive)=" << oldCmin0-cmin0 << "; " << oldCmin1-cmin1 <<std::endl;

					m_files->OutputInd(candidate,
						m_pop->GetInd(i),
						m_probl->GetNumEval(),
						domination);

					if (usedFESthisLS>=5000 || nfeval>=m_params->GetMaxEval() /*|| 
						(currentFES_LS+usedFESthisLS)/ m_probl->GetNumEval() < ratioFES_LS*/) break;
				}
				currentFES_LS+=usedFESthisLS;
			}
			//konec LS (SQP)
		} else if (m_params->strategyPrunning==2) {
			int variantS2=1;
			for (UINT i = 0; i < variantS2==0?m_params->GetPopSize()/5:m_params->GetPopSize(); i++) {
				int oldi=i;
				int index = int((double) rand() / RAND_MAX*m_pop->GetInds().size());
				i=index;
				if (m_pop->GetInd(index)->GetDomination()==g_domin_first &&
					(double) rand() / RAND_MAX < 0.8 || (double) rand() / RAND_MAX < 0.5) {
					DemoInd *candidate = new DemoInd();
					double F=m_params->GetWeight(),CR=m_params->GetCrossProb();//patch by AlesZ
					candidate->SetGenes(m_pop->GetCandidate(i, F,CR,candidate,nondominatedInd));//patch by AlesZ
					candidate->SetWeightF(F);//if strategy != 13, not importaint
					candidate->SetCrossProbCR(CR);//if strategy != 13, not importaint
					candidate->Evaluate(m_probl);

					domination = m_probl->GetDomination(candidate, m_pop->GetInd(i));
					candidate->SetDomination( domination);
					m_files->OutputInd(candidate,
														m_pop->GetInd(i),
														m_probl->GetNumEval(),
														domination);

					if (m_always_add) {
						m_pop->AppendInd(candidate);
					}
					else {
						if (domination == g_domin_first) {
							*m_pop->GetInd(i) = *candidate;
							delete candidate;
						}
						else if (domination == g_domin_second) {
							delete candidate;
						}
						else if (domination == g_domin_none) {
							m_pop->AppendInd(candidate);
						}
					}
				} else oldi--;
				i=oldi;
			}
			indexes = m_pop->SortPop();
			m_pop->TruncPop(indexes);
		} else if (m_params->strategyPrunning==3 || true) {
			//80%SPEA,20% IBEA
			for (UINT i = 0; i < m_params->GetPopSize(); i++) {
				DemoInd *candidate = new DemoInd();
				double F=m_params->GetWeight(),CR=m_params->GetCrossProb();//patch by AlesZ
				candidate->SetGenes(m_pop->GetCandidate(i, F,CR,candidate,nondominatedInd));//patch by AlesZ
				candidate->SetWeightF(F);//if strategy != 13, not importaint
				candidate->SetCrossProbCR(CR);//if strategy != 13, not importaint
				candidate->Evaluate(m_probl);

				domination = m_probl->GetDomination(candidate, m_pop->GetInd(i));
				candidate->SetDomination( domination);
				m_files->OutputInd(candidate,
													m_pop->GetInd(i),
													m_probl->GetNumEval(),
													domination);

				if (m_always_add) {
					m_pop->AppendInd(candidate);
				}
				else {
					if (domination == g_domin_first) {
						*m_pop->GetInd(i) = *candidate;
						delete candidate;
					}
					else if (domination == g_domin_second) {
						delete candidate;
					}
					else if (domination == g_domin_none) {
						m_pop->AppendInd(candidate);
					}
				}
			}

			//PRUNNING
			if ((double) rand() / RAND_MAX > 0.2) {
				indexes = m_pop->SortPop();
				m_pop->TruncPop(indexes);
			}
			else {//in 20%, execute IBEA
				//load
				vector<UINT> empty;
				m_pop_ibea->TruncPop(empty);//delete inds & throw away pointers
				vector<DemoInd*>& inds_pop = m_pop->GetInds();
				for (int i = 0; i < inds_pop.size(); i++) {
					m_pop_ibea->AppendInd(new DemoInd(*inds_pop[i]));
				}

				//process
				indexes = m_pop_ibea->SortPop();
				m_pop_ibea->TruncPop(indexes);//do prunning, delete unnecessary

				///save
				//destroy all old individuals
				m_pop->TruncPop(empty);
				vector<DemoInd*>& inds_pop_ibea = m_pop_ibea->GetInds();
				for (int i = 0; i < inds_pop_ibea.size(); i++) {
					//add deep copies, because originals get destroyed at
					//load time of this strategy for TruncPop of IBEA call
					m_pop->AppendInd(new DemoInd(*inds_pop_ibea[i]));
				}
			}
		}

		updateEpsilonConstraint();
//		m_files->OutputPop(m_pop, (m_probl->GetNumEval() / m_params->GetPopSize()));
		
		///create temporary Inds storage, because they might get deleted before printout
		vector<DemoInd*> m_appSet_copy;
		for (int i = 0; i < (int)m_appSet.size(); i++)
			m_appSet_copy.push_back(new DemoInd(*m_appSet[i]));
		m_files->OutputPop(m_pop, 
			(m_probl->GetNumEval() / m_params->GetPopSize()),
			&m_appSet_copy);

	}
}
//---------------------------------------------------------------------------

void DemoAlg::savePopulationToLongApproximationSet() {
	std::cout <<std::endl << "444 save start: " <<std::flush;
//	system("date");
	vector<DemoInd*> m_inds;
	m_probl = m_pop->GetProbl();

	///odstranim dominirane v trenutni populaciji, ki ji zelim dodajati v APP
	for (int i = 0; i < m_pop->GetInds().size(); i++) {
		bool dominated=false;
		for (int j = 0; j < m_pop->GetInds().size(); j++) {
			if (i == j) continue;
			int dominate = m_probl->GetDomination(m_pop->GetInd(i), m_pop->GetInd(j));
			if (dominate == g_domin_second) {
				dominated=true;
				break;
			}
		}
		if (!dominated)
			m_inds.push_back(m_pop->GetInd(i));
	}

	vector<DemoInd*>::const_iterator cii;
	int APS=m_appSet.size();//200*800=160.000

	///skopiram izbrane resitve v APP, prepisujem starejse (hranim min. 20G 2*NP)?
	///pozor: m_appSet se kasneje lahko tudi zmanjsa!!!
	bool isAPPSlistAround=false;

// for (int i = 0; i < m_inds.size(); i++) {
// 		for (int j = 0; j < m_probl->GetNumFunc(); j++)
// 			cout << i << "@" << j << "=" << m_inds[i]->GetCriterion(j)<<endl;
// 	}
// 
// cout <<"m_appSet_currInd="<<m_appSet_currInd<<endl;
// cout << "m_appSet.size()" << m_appSet.size()<<endl;
// 	for (int i = 0; i < m_appSet.size(); i++) {
// 		for (int j = 0; j < m_probl->GetNumFunc(); j++)
// 			cout << i << "@" << j << "=" << m_appSet[i]->GetCriterion(j)<<endl;
// 	}
// 


	for(cii=m_inds.begin(); cii!=m_inds.end(); cii++) {
		///HACK: copy by value!
		delete m_appSet[m_appSet_currInd];
		m_appSet[m_appSet_currInd]=0;
		m_appSet[m_appSet_currInd]=new DemoInd(**cii);
		m_appSet_currInd=(m_appSet_currInd+1)%APS;
		if (!m_appSet_currInd)
			isAPPSlistAround=true;
	}

	///se enkrat vse skozi -- dominance
	int look_toNP=isAPPSlistAround?APS:m_appSet_currInd;

	m_inds.clear();
	static DemoInd *tmpArch = new DemoInd[APS];
	int currArSz=0;
	for (int i = 0; i < look_toNP; i++) {
		bool dominated=false;
		for (int j = 0; j < look_toNP; j++) {
			if (i == j) continue;
			int dominate = m_probl->GetDomination(m_appSet[i], m_appSet[j]);
			if (dominate == g_domin_second){
				dominated=true;
				break;
			}
		}
		if (!dominated)
			tmpArch[currArSz++]=*m_appSet[i];
	}

	///velikost APP -- size of ( m_appSet ) = look_toNP => currArSz
	for (int i = 0; i < currArSz; i++)
		*m_appSet[i]=tmpArch[i];
	m_appSet_currInd=currArSz%APS;

	std::cout << "444 save   end: " <<std::flush;
//	system("date");
	std::cout << "APPSZnew="<<currArSz<<std::endl;
}


void DemoAlg::OutputInfoGen()
{
	if (!m_files->GetFileGen()->IsOff()) {
		stringstream tmp;
		tmp << endl << "gen_number" << "\t";
		tmp << "num_feasible" << "\t";
		tmp << "num_unfeasible" << "\t";
		tmp << "num_non_evaluated" << "\t";
		tmp << "avg_violation" << "\t";
		tmp << "hyp_feasible" << "\t";
		tmp << "first_front_size" << "\t";
		tmp << "c_better" << "\t";
		tmp << "p_better" << "\t";
		tmp << "incomparable";
		m_files->GetFileGen()->Output(tmp.str());
	}
}
//---------------------------------------------------------------------------

void DemoAlg::OutputInfoLog()
{
	if (!m_files->GetFileLog()->IsOff()) {
		stringstream tmp;
		tmp << "Information on this run:" << endl;
		tmp << "\tseed\t" << m_seed << endl;
		tmp << endl << "Paramters of the algorithm:" << endl;
		tmp << "\tpop_size\t" << m_params->GetPopSize() << endl;
		tmp << "\tmax_eval\t" << m_params->GetMaxEval() << endl;
		tmp << "\tweight\t" << m_params->GetWeight() << endl;
		tmp << "\tcross_prob\t" << m_params->GetCrossProb() << endl;
		tmp << "\talways_add\t" << GetBoolStringYN(m_always_add) << endl;
		tmp << "\tsel_type\t";
		if (m_sel_type == g_sel_NSGA) {
			tmp << "NSGA-II" << endl;
			tmp << "\telitism\t" << GetBoolStringYN(m_pop_nsga->GetElitism()) << endl;
			tmp << "\telitism_r\t" << m_pop_nsga->GetElitismR() << endl;
		}
		else if (m_sel_type == g_sel_IBEA) {
			tmp << "IBEA" << endl;
			tmp << "\tindicator\t";
			if (m_pop_ibea->GetIndicator() == g_ibea_eps)
				tmp << "eps" << endl;
			else if (m_pop_ibea->GetIndicator() == g_ibea_hyp)
				tmp << "hyp" << endl;
			tmp << "\trho\t" << m_pop_ibea->GetRho() << endl;
			tmp << "\tkappa\t" << m_pop_ibea->GetKappa() << endl;
		}
		else if (m_sel_type == g_sel_SPEA) {
			tmp << "SPEA" << endl;
		}

		tmp << endl << "Paramters of the problem:" << endl;
		tmp << "\tfunc_type\t" << m_probl->GetProblemString() << endl;
		tmp << "\tnum_var\t" << m_probl->GetNumVar() << endl;
		tmp << m_probl->GetVariableString();
		tmp << "\tnum_func\t" << m_probl->GetNumFunc() << endl;
		tmp << m_probl->GetFunctionString();
		tmp << "\tnum_feat\t" << m_probl->GetNumFeat() << endl;
		if (m_probl->GetFunction() == g_prob_extern) {
			tmp << "\tcommand\t" << m_probl_sim->GetCommand() << endl;
			tmp << "\tarchive\t" << m_probl_sim->GetArchive() << endl;
			tmp << "\tarchive_read\t" << GetBoolStringYN(m_probl_sim->GetArchiveRead()) << endl;
			tmp << "\tarchive_write\t" << GetBoolStringYN(m_probl_sim->GetArchiveWrite()) << endl;
			tmp << "\tsim_in\t" << m_probl_sim->GetSimIn() << endl;
			tmp << "\tsim_out\t" << m_probl_sim->GetSimOut() << endl;
		}
		else if ((m_probl->GetFunction() >= g_prob_fun_WFG1) &&
						 (m_probl->GetFunction() <= g_prob_fun_WFG9)) {
			tmp << "\twfg_M\t" << m_probl_fun->GetWfgM() << endl;
			tmp << "\twfg_k\t" << m_probl_fun->GetWfgK() << endl;
			tmp << "\twfg_l\t" << m_probl_fun->GetWfgL() << endl;
		}

		tmp << endl << "Parameters for the output:" << endl;
		tmp << "\tlog_file_name\t" << m_files->GetFileLog()->GetName() << endl;
		tmp << "\tlog_mode\t" << m_files->GetFileLog()->GetModeString() << endl;
		tmp << "\tfront_file_name\t" << m_files->GetFileFront()->GetName() << endl;
		tmp << "\tfront_mode\t" << m_files->GetFileFront()->GetModeString() << endl;
		tmp << "\tfront_gen\t" << m_files->GetFrontGen() << endl;
		tmp << "\tfront_long\t" << GetBoolStringYN(m_files->GetFrontLong()) << endl;
		tmp << "\trepl_file_name\t" << m_files->GetFileGen()->GetName() << endl;
		tmp << "\trepl_mode\t" << m_files->GetFileGen()->GetModeString() << endl;
		tmp << "\tlines_file_name\t" << m_files->GetFileLines()->GetName() << endl;
		tmp << "\tlines_mode\t" << m_files->GetFileLines()->GetModeString() << endl;
		tmp << endl;

		m_files->GetFileLog()->Output(tmp.str());
	}
}
//---------------------------------------------------------------------------

void DemoAlg::TimingsCec07() {
	const int NR_CEC07_FUNCS=19;

	ProblFunCec07 cec07fun[NR_CEC07_FUNCS];
	for (UINT i = 0; i < NR_CEC07_FUNCS; i++) {
		cec07fun[i].SetFunction(101+i);
		cec07fun[i].InitProblem();
	}

	vector<double> maxCrits;
	maxCrits.resize(5);

	DemoInd ind[NR_CEC07_FUNCS];
	for (UINT i = 0; i < NR_CEC07_FUNCS; i++) {
		ind[i].SetGenes(cec07fun[i].GetRandVector());
		ind[i].SetCriteria(maxCrits);
	}

	//evaluate
	cout << "Start (T1) >>"<<endl;
	system("date +%F-%T-%N");
	for (UINT i = 0; i < NR_CEC07_FUNCS; i++) {
		for (UINT j = 0; j < 10000; j++) {
			ind[i].Evaluate(&cec07fun[i]);
		}
	}
	system("date +%F-%T-%N");
	cout << "<<Finish"<<endl;

	m_files->SetFrontGen(0);
	m_files->SetFrontLong(0);
	m_params->SetMaxEval(10000);
	for (UINT i = 0; i < NR_CEC07_FUNCS; i++) {
		cec07fun[i].InitProblem();
	}
/*
	int cec07Popsizes[]={
100,
100,
100,
100,
100,
100,
100,
150,
800,
150,
800,
150,
800,
150,
800,
150,
800,
150,
800};

	cout << "Running time (T2)>>"<<endl;
	system("date +%F-%T-%N");
	for (UINT k = 0; k < NR_CEC07_FUNCS; k++) {
		m_params->SetPopSize(cec07Popsizes[k]);
		m_pop->InitPop(m_params, &cec07fun[k]);

		m_pop->CreateFirstPop();
		m_pop->EvaluatePop(m_files);
		m_files->OutputPop(m_pop, 1, m_appSet);

		vector<UINT> indexes;
		UINT domination;
		while (cec07fun[k].GetNumEval() < m_params->GetMaxEval()) {

			indexes = m_pop->SortPopRandom();
			m_pop->TruncPop(indexes);//time consuming: make a deep rearanged copy of individuals
			vector<int> nondominatedInd;//Nondominated indices of individuals
			generateNDList(nondominatedInd);

			for (UINT i = 0; i < m_params->GetPopSize(); i++) {
				DemoInd *candidate = new DemoInd();
				double F=m_params->GetWeight(),CR=m_params->GetCrossProb();//patch by AlesZ
				candidate->SetGenes(m_pop->GetCandidate(i, F,CR,candidate,nondominatedInd));//patch by AlesZ
				candidate->SetWeightF(F);//if strategy != 13, not importaint
				candidate->SetCrossProbCR(CR);//if strategy != 13, not importaint
				candidate->Evaluate(&cec07fun[k]);

				domination = cec07fun[k].GetDomination(candidate, m_pop->GetInd(i));
				m_files->OutputInd(candidate,
						m_pop->GetInd(i),
								cec07fun[k].GetNumEval(),
										domination);

				if (m_always_add) {
					m_pop->AppendInd(candidate);
				}
				else {
					if (domination == g_domin_first) {
						*m_pop->GetInd(i) = *candidate;
						delete candidate;
					}
					else if (domination == g_domin_second) {
						delete candidate;
					}
					else if (domination == g_domin_none) {
						m_pop->AppendInd(candidate);
					}
				}

			}
			indexes = m_pop->SortPop();
			m_pop->TruncPop(indexes);

			m_files->OutputPop(m_pop, (cec07fun[k].GetNumEval() / m_params->GetPopSize()), 1);
		}
	}
	system("date +%F-%T-%N");
	cout << "<<Finish (T2)"<<endl;
*/
	return;
}
//---------------------------------------------------------------------------

void DemoAlg::generateNDList(vector<int> &nondominatedInd) {
// TODO: ni prav, znova!
		for (UINT i = 0; i < m_params->GetPopSize(); i++) {
		int domination = m_pop->GetInd(i)->GetDomination();
		if (domination == g_domin_none) {//if nondominated
			nondominatedInd.push_back(i);
		}
	}
}

//---------------------------------------------------------------------------
void DemoAlg::initEpsilonConstraint() {
	//sort by violation
	std::vector<double> epsilons;
	//extract data
	for (int i = 0; i < m_params->GetPopSize(); i++)
		epsilons.push_back(m_pop->GetInd(i)->GetViolation());

	//sort epsilons, ascending
	sort(epsilons.begin(), epsilons.end());

	//then take the 30%-th
	int epsInd=int(0.7*m_params->GetPopSize());
	vEpsilon=epsilons[epsInd];

	m_probl->epsilonConstraint = vEpsilon;
}

void DemoAlg::updateEpsilonConstraint() {
	const double percFESconstr=0.2;
	const double Cp=5;
	const double minEPSavg=1e-06;
	const double alpha2=2;
	const double alpha1=0.8;

	double Gperc=float(m_probl->GetNumEval()) / float(m_params->GetMaxEval());
	double GconstrPerc=Gperc/percFESconstr;
	double newEpsilon=minEPSavg;

	//only first few percents, the epsilon constraint is relaxed
	if (Gperc < percFESconstr) {
		//calculate vBeta
	        //sort by violation
        	std::vector<double> epsilons;
	        //extract data
        	for (int i = 0; i < m_params->GetPopSize(); i++)
                	epsilons.push_back(m_pop->GetInd(i)->GetViolation());

	        //sort epsilons, ascending
        	sort(epsilons.begin(), epsilons.end());

	        //then take the 70%-th
        	int epsInd=int(0.3*m_params->GetPopSize());//=top 70%-th
	        double vBeta=epsilons[epsInd];

		//adjust the vEpsilon coefficient
		if (alpha2 * vBeta < vEpsilon)//if feasibility improved by 2 (=alpha2) for the top 70%-th individual
			vEpsilon=alpha1*vEpsilon;

		double epsDiminished=vEpsilon*pow(1-GconstrPerc,Cp);
		if (epsDiminished > minEPSavg)
			m_probl->epsilonConstraint = epsDiminished;
	}
}

