/*===========================================================================

	Copyright  2007 Tea Tuar

	This material is provided "as is", with no warranty expressed or implied.
	Any use is at your own risk. Permission to use or copy this software for
	any purpose is hereby granted without fee, provided this notice is
	retained on all copies. Permission to modify the code and to distribute
	modified code is granted, provided a notice that the code was modified is
	included with the above copyright notice.

	http://dis.ijs.si/tea/research.htm

	===========================================================================

	Header file.

	Defines the class DemoAlg.

	file: demo_alg.h
	author: Tea Tusar (tea.tusar@ijs.si)
	created: 17.02.2006
	last change: $date$

	===========================================================================
*/

#ifndef demo_algH
#define demo_algH
//---------------------------------------------------------------------------

#include "demo_params.h"
#include "demo_utilities.h"
#include "probl.h"
#include "probl_fun.h"
#include "problfuncec07.h"
#include "problfuncec09.h"
#include "cec09.h"
#include "probl_sim.h"
#include "demo_ind.h"
#include "demo_pop.h"
#include "demo_pop_nsga.h"
#include "demo_pop_ibea.h"
#include "demo_pop_spea.h"
#include "file_stat.h"

class DemowsaLocalsearch;
//---------------------------------------------------------------------------

/** Selection types. */
extern const UINT g_sel_NSGA;
extern const UINT g_sel_IBEA;
extern const UINT g_sel_SPEA;
//---------------------------------------------------------------------------

class DemoAlg
{
	/** Parameters. */
	DemoParams *m_params;

	/** Files. */
	FileStat *m_files;

	/** Problem. */
	Probl *m_probl;

	/** Problem using test functions for evaluation. */
	ProblFun *m_probl_fun;

	/** Problem using simulator for evaluation. */
	ProblSim *m_probl_sim;

	/** Population. */
	DemoPop *m_pop;

	/** Population using IBEA's selection. */
	DemoPopIbea *m_pop_ibea;

	/** Population using NSGA-II's selection. */
	DemoPopNsga *m_pop_nsga;

	/** Population using SPEA2's selection. */
	DemoPopSpea *m_pop_spea;

	/** Seed of the random generator. */
	UINT m_seed;

	/** Selection type. */
	UINT m_sel_type;

	/** Whether to always add the candidates or to add only the candidates that
	are not dominated by their parents. */
	bool m_always_add;

	/** Added by AlesZ: create timings for 10000 executions of test funcitons (e.g. for CEC07)*/
	int timingsOnly;

	/** Vector of individuals for approximation set calculation. */
	vector<DemoInd*> m_appSet;

	int m_appSet_currInd;


	DemowsaLocalsearch *m_DemowsaLocalsearch;

public:
	/** Class constructor.
	throws: exception if the input file cannot be opened for reading
	*/
	DemoAlg(string ini_f_name, string ini_args);

	/** Class destructor. */
	~DemoAlg();

	/** Function opens the input file stream for reading.
	params: f_name file name of the input file stream
	*/
	void ReadFromFile(string f_name);

	/** Function stores the input line arguements into a stream for reading.
	params: args string containing input line arguements
	*/
	void ReadFromString(string args);

	/** Function reads the input parameters from the given input stream. Each
	parameter is checked - if the input value is incorrect, an exception is
	thrown.
	params: stream input stream
	throws: exception if an input value is not correct
	*/
	void ReadFromStream(istream &stream);

	/** Initialization function initializes all members with the data read from
	a stream. */
	void InitAlg();

	/** Function starts the evolution by creating the first population and then
	creating new candidates until the stopping criterion (number of evaluated
	individuals) is met. */
	void Evolution();

	/** Function outputs initial information into the generation file. */
	void OutputInfoGen();

	/** Function outputs initial information into the log file. */
	void OutputInfoLog();

	/** Debugging functions for use, when the programme crashes -- added by AlesZ <ales.zamuda@uni-mb.si>*/
	inline DemoParams* GetParamsForDebug() {return m_params;}
	inline Probl* GetProblForDebug() {return m_probl;}
	inline UINT GetSeedForDebug() {return m_seed;}

protected:
	/** Prints timings for Cec07 and exits*/
	void TimingsCec07();

	/** Generates list of non-dominated indices*/
	void generateNDList(vector<int> &nondominatedInd);

	/** Saves last few generations in a separate approximation set.*/
	void savePopulationToLongApproximationSet();

	void checkCandidate(DemoInd &candidate);

	void initEpsilonConstraint();
	void updateEpsilonConstraint();
	double vEpsilon;
};
//---------------------------------------------------------------------------
#endif

