/*===========================================================================

	Copyright  2007 Tea Tuar

	This material is provided "as is", with no warranty expressed or implied.
	Any use is at your own risk. Permission to use or copy this software for
	any purpose is hereby granted without fee, provided this notice is
	retained on all copies. Permission to modify the code and to distribute
	modified code is granted, provided a notice that the code was modified is
	included with the above copyright notice.

	http://dis.ijs.si/tea/research.htm

	===========================================================================

	Implementation file.

	file: demo_ind.cpp
	author: Tea Tusar (tea.tusar@ijs.si)
	created: 16.02.2006
	last change: $date$

	===========================================================================
*/

#include "demo_ind.h"
//---------------------------------------------------------------------------

void DemoInd::SetGenes(vector<double> genes)
	{m_genes = genes;}

void DemoInd::SetCriteria(vector<double> criteria)
	{m_criteria = criteria;}

void DemoInd::SetFeatures(vector<double> features)
	{m_features = features;}

void DemoInd::SetViolation(double violation)
	{m_violation = violation;}

void DemoInd::SetEvaluated(bool evaluated)
	{m_evaluated = evaluated;}
//---------------------------------------------------------------------------

DemoInd::DemoInd()
{
	m_genes.clear();
	m_criteria.clear();
	m_features.clear();
	m_violation = 0;
	m_evaluated = false;
	//these, F & CR, are used for self-adaptive InitPop&GetCandidate
        m_WeightF = /*0.3+0.4**/(double) rand() / RAND_MAX;
	m_CrossProbCR = /*0.3+0.2**/(double) rand() / RAND_MAX;
	m_domination=g_domin_none;
}
//---------------------------------------------------------------------------

DemoInd::~DemoInd()
{
}
//---------------------------------------------------------------------------

DemoInd::DemoInd(DemoInd& ind)
{
	m_genes = ind.m_genes;
	m_criteria = ind.m_criteria;
	m_features = ind.m_features;
	m_violation = ind.m_violation;
	m_evaluated = ind.m_evaluated;
	m_WeightF = ind.m_WeightF;
	m_CrossProbCR = ind.m_CrossProbCR;
	m_WeightFVect = ind.m_WeightFVect;
	m_CrossProbCRVect = ind.m_CrossProbCRVect;
	m_domination = ind.m_domination;
}
//---------------------------------------------------------------------------

DemoInd& DemoInd::operator=(const DemoInd& ind)
{
	if (this != &ind) {
		m_genes = ind.m_genes;
		m_criteria = ind.m_criteria;
		m_features = ind.m_features;
		m_violation = ind.m_violation;
		m_evaluated = ind.m_evaluated;
		m_WeightF = ind.m_WeightF;
		m_CrossProbCR = ind.m_CrossProbCR;
		m_WeightFVect = ind.m_WeightFVect;
		m_CrossProbCRVect = ind.m_CrossProbCRVect;
		m_domination = ind.m_domination;
	}
	return *this;
}
//---------------------------------------------------------------------------

void DemoInd::Evaluate(Probl *probl)
{
	probl->IncNumEval();
	probl->Evaluate(this);
}
//---------------------------------------------------------------------------

string DemoInd::GetIndString()
{
	UINT i;
	stringstream result;

	result.precision(g_out_prec);
	for (i = 0; i < m_genes.size(); i++)
		result << scientific << m_genes.at(i) << "\t";
	result << GetBoolString01(m_evaluated) << "\t";
	result << scientific << m_violation << "\t";
	for (i = 0; i < m_criteria.size(); i++)
		result << scientific << m_criteria.at(i) << "\t";
	for (i = 0; i < m_features.size(); i++)
		result << scientific << m_features.at(i) << "\t";
	result << endl;

	return result.str();
}
//---------------------------------------------------------------------------

string DemoInd::GetIndCritString()
{
	UINT i;
	stringstream result;

	result.precision(g_out_prec);
	for (i = 0; i < m_criteria.size(); i++)
		result << scientific << m_criteria.at(i) << "\t";
	for (i = 0; i < m_constraints.size(); i++)
		result << scientific << m_constraints.at(i) << "\t";

	result << endl;

	return result.str();
}
//---------------------------------------------------------------------------

