/*===========================================================================

	Copyright  2007 Tea Tuar

	This material is provided "as is", with no warranty expressed or implied.
	Any use is at your own risk. Permission to use or copy this software for
	any purpose is hereby granted without fee, provided this notice is
	retained on all copies. Permission to modify the code and to distribute
	modified code is granted, provided a notice that the code was modified is
	included with the above copyright notice.

	http://dis.ijs.si/tea/research.htm

	===========================================================================

	Header file.

	Defines the class DemoInd.

	file: demo_ind.h
	author: Tea Tusar (tea.tusar@ijs.si)
	created: 16.02.2006
	last change: $date$

	===========================================================================
*/

#ifndef demo_indH
#define demo_indH
//---------------------------------------------------------------------------

#include "demo_params.h"
#include "demo_utilities.h"
#include "probl.h"
//---------------------------------------------------------------------------

class Probl;
//---------------------------------------------------------------------------

class DemoInd
{
	/** Vector of variables. */
	vector<double> m_genes;

	/** Vector of criteria. If the individual cannot be evaluated, the criteria
	must consist of some high values that are read from the simulator's output
	file. */
	vector<double> m_criteria;
	
	/** Vector of constraints. */
	vector<double> m_constraints;

	/** Vector of features. Features are used for getting some additional
	information on the evaluated individual. The number of features is fixed
	and depends on the problem. If a simulator is used for calculating the
	criteria, the features are read after the criteria. */
	vector<double> m_features;

	/** Violation value. If the individual violates the constraints, this value
	is greater than zero. */
	double m_violation;

	/** A boolean value indicating whether the individual has been properly
	evaluated. */
	bool m_evaluated;

	/** Wether the individual was dominated in the parent population */
	int m_domination;

	/** Individual level F*/
	double m_WeightF;

	/** Individual level CR*/
	double m_CrossProbCR;

	/** Individual level F -- array */
	vector<double> m_WeightFVect;
	/** Individual level CR -- array */
	vector<double> m_CrossProbCRVect;

public:
	void SetGenes(vector<double> genes);
	void SetCriteria(vector<double> criteria);
	inline void SetConstraints(vector<double> &constraints) {m_constraints=constraints;}
	void SetFeatures(vector<double> features);
	void SetViolation(double violation);
	void SetEvaluated(bool evaluated);
	inline void SetDomination(int domination) {m_domination=domination;}
	inline double SetWeightF(double weightF) {m_WeightF = weightF;}
	inline double SetCrossProbCR(double crossProbCR) {m_CrossProbCR = crossProbCR;}

	inline void SetCriterion(UINT index, double criterion)
		{m_criteria.at(index) = criterion;};

	inline vector<double>& GetGenes() {return m_genes;}
	inline double GetGene(UINT index) {return m_genes[index];}
	inline void SetGene(UINT index, double value) {m_genes[index]=value;}
	inline vector<double>& GetCriteria() {return m_criteria;}
	inline double GetCriterion(UINT index) {return m_criteria[index];}
	inline vector<double>& GetFeatures() {return m_features;}
	inline vector<double>& GetConstraints() {return m_constraints;}
	inline double GetFeature(UINT index) {return m_features[index];}
	inline double GetViolation() {return m_violation;}
	inline bool GetEvaluated() {return m_evaluated;}
	inline bool GetDomination() {return m_domination;}
	inline double GetWeightF() {return m_WeightF;}
	inline double GetCrossProbCR() {return m_CrossProbCR;}
	inline vector<double>& GetWeightFVect() {return m_WeightFVect;}
	inline vector<double>& GetCrossProbCRVect() {return m_CrossProbCRVect;}

	/** Class constructor. */
	DemoInd();

	/** Class destructor. */
	~DemoInd();

	/** Copy constructor. */
	DemoInd(DemoInd& ind);

	/** Copy operator. */
	DemoInd& operator=(const DemoInd& ind);

	/** Function calls the evaluation of the current individual and increases the
	number of evaluated individuals.
	params: probl points to the handled problem */
	void Evaluate(Probl *probl);

	/** Function builds and returns a string with information of the individual.
	returns: the built string */
	string GetIndString();

	/** Function builds and returns a string with individual's criteria
	returns: the built string */
	string GetIndCritString();
};
//---------------------------------------------------------------------------
#endif


