/*===========================================================================

	Copyright  2007 Tea Tuar

	This material is provided "as is", with no warranty expressed or implied.
	Any use is at your own risk. Permission to use or copy this software for
	any purpose is hereby granted without fee, provided this notice is
	retained on all copies. Permission to modify the code and to distribute
	modified code is granted, provided a notice that the code was modified is
	included with the above copyright notice.

	http://dis.ijs.si/tea/research.htm

	===========================================================================

	Header file.

	Defines the class DemoParams.

	file: demo_params.h
	author: Tea Tusar (tea.tusar@ijs.si)
	created: 16.02.2006
	last change: $date$

	===========================================================================
*/

#ifndef demo_paramsH
#define demo_paramsH
//---------------------------------------------------------------------------

#include <math.h>
#include <values.h>
#include <vector>
#include <list>
#include <string>
#include <sstream>
#include <fstream>
#include <iomanip>
#include <iostream>

using namespace std;
typedef unsigned int UINT;
//---------------------------------------------------------------------------

class DemoParams
{
private:
	/** Number of individuals in the population. */
	UINT m_pop_size;

	/** Number of maximum allowed evaluations. */
	UINT m_max_eval;

	/** Crossover probability. */
	double m_cross_prob;

	/** Weight used in individual creation. */
	double m_weight;

public:
	/** Constraints for self-adaptive control parameters */
	double minF;
	double maxF;
	double minCR;
	double maxCR;
	double learnTau;
	int strategyDE;
	int strategyPrunning;
	int appsetMaxGenerations;
	double NP_size_factor;//for  how much NP is larger than the output approximation set
	int approximationSetSize;

public:
	inline void SetPopSize(UINT pop_size) {m_pop_size = pop_size;};
	inline void SetMaxEval(UINT max_eval) {m_max_eval = max_eval;};
	inline void SetCrossProb(double cross_prob) {m_cross_prob = cross_prob;};
	inline void SetWeight(double weight) {m_weight = weight;};

	inline UINT GetPopSize() {return m_pop_size;};
	inline UINT GetMaxEval() {return m_max_eval;};
	inline double GetCrossProb() {return m_cross_prob;};
	inline double GetWeight() {return m_weight;};

	/** Class constructor. */
	DemoParams();

	/** Copy constructor. */
	DemoParams(DemoParams& params);

	/** Class destructor. */
	~DemoParams() {};
};
//---------------------------------------------------------------------------
#endif


