/*
	Modified code version of the file:
		Copyright 2007 Ales Zamuda <ales.zamua@uni-mb.si>
		Copyright 2007 Janez Brest <janez.brest@uni-mb.si>
		License: LGLP

	Copyright notice from IJS:
	===========================================================================

	Copyright  2007 Tea Tusar

	This material is provided "as is", with no warranty expressed or implied.
	Any use is at your own risk. Permission to use or copy this software for
	any purpose is hereby granted without fee, provided this notice is
	retained on all copies. Permission to modify the code and to distribute
	modified code is granted, provided a notice that the code was modified is
	included with the above copyright notice.

	http://dis.ijs.si/tea/research.htm

	===========================================================================

	Implementation file.

	file: demo_pop.cpp
	author: Tea Tusar (tea.tusar@ijs.si)
	created: 17.02.2006
	last change: $date$

	===========================================================================
*/

#include "demo_pop.h"
#include "demo_utilities.h"
#include "probl.h"
//added by AlesZ
#include <gsl/gsl_rng.h>
#include <gsl/gsl_randist.h>
//---------------------------------------------------------------------------

gsl_rng * DemoPop::rNormalDistr=0;

DemoPop::DemoPop()
{
	if (rNormalDistr==0) {
		const gsl_rng_type * T;
		gsl_rng_env_setup();
		T = gsl_rng_default;
		rNormalDistr = gsl_rng_alloc ( T );
	}
}
//---------------------------------------------------------------------------

DemoPop::~DemoPop()
{
	for (UINT i = 0; i < m_inds.size(); i++)
		delete m_inds.at(i);
}
//---------------------------------------------------------------------------

void DemoPop::InitPop(DemoParams *params, Probl *probl)
{
	m_params = params;
	m_probl = probl;

	m_inds.clear();
	for (UINT i = 0; i < m_params->GetPopSize(); i++) {
		DemoInd *ind = new DemoInd();
		m_inds.push_back(ind);
	}
}
//---------------------------------------------------------------------------

void DemoPop::CreateFirstPop()
{
	for (UINT i = 0; i < m_params->GetPopSize(); i++)
		m_inds.at(i)->SetGenes(m_probl->GetRandVector());
}
//---------------------------------------------------------------------------

void DemoPop::EvaluatePop(FileStat *files)
{
	for (UINT i = 0; i < m_inds.size(); i++) {
		m_inds.at(i)->Evaluate(m_probl);
		files->OutputInd(m_inds.at(i), NULL, m_probl->GetNumEval(), g_domin_original);
	}
}
//---------------------------------------------------------------------------

/**
 * Repairs new F or CR to be in [0,1]
 * @param number F or CR, by reference
 * @param min min border
 * @param max max border
 */
void repairF_CRNumber(double *number, double min, double max) {
	//any other strategy can be used here:
	//-boundary mirror
	//-boundery mirror, not over half
	//-adding N(0,1) until number fits the borders
	if ( *number < min )
		*number = min;
	if ( *number > max )
		*number = max;
}

vector<double> DemoPop::GetCandidate(UINT parent, double &F, double &CR, DemoInd *candidate, const vector<int> &nondominatedInd)
{
	UINT i1, i2, i3, i4;
	UINT i, i_rand;
	vector<double> result;
	result.clear();
	for (i = 0; i < m_probl->GetNumVar(); i++)
		result.push_back(0);

	do i1 = random(m_inds.size());
	while (i1 == parent);

	do i2 = random(m_inds.size());
	while ((i2 == parent) || (i2 == i1));

	do i3 = random(m_inds.size());
	while ((i3 == parent) || (i3 == i1) || (i3 == i2));

	do i4 = random(m_inds.size());
	while ((i4 == parent) || (i4 == i1) || (i4 == i2) || (i4 == i3));

	int selDESubStrategy=0; // selected sub-strategy
	DemoInd* bestit=candidate;//HACK: impossible to implement: almost all individuals are BEST (Pareto-efficient)

	if (!nondominatedInd.empty()) {
		int chosenBest=random(nondominatedInd.size());
		bestit=m_inds.at(nondominatedInd[chosenBest]);
	}

	if (m_params->strategyDE==13 || m_params->strategyDE==14) {
		double Fi,F1,F2,F3;
		double CRi,CR1,CR2,CR3;

		Fi=m_inds[parent]->GetWeightF();
		F1=m_inds[i1]->GetWeightF();
		F2=m_inds[i2]->GetWeightF();
		F3=m_inds[i3]->GetWeightF();

		CRi=m_inds[parent]->GetCrossProbCR();
		CR1=m_inds[i1]->GetCrossProbCR();
		CR2=m_inds[i2]->GetCrossProbCR();
		CR3=m_inds[i3]->GetCrossProbCR();
		double tau=1.0/(m_params->learnTau*sqrt(2*m_probl->GetNumVar()));

		F = (Fi+F1+F2+F3)/4 *
				pow(M_E,tau*  gsl_ran_gaussian( rNormalDistr, 1 ) );
		CR = (CRi+CR1+CR2+CR3)/4 *
				pow(M_E,tau* gsl_ran_gaussian( rNormalDistr, 1 ) );

		repairF_CRNumber(&F, m_params->minF, m_params->maxF);
		repairF_CRNumber(&CR, m_params->minCR, m_params->maxCR);
	} else if (m_params->strategyDE==107) {//from jDE (Brest et al.) => DEMOwSA//jDE
		const double Fl=0.1,Fu=0.9;
		const double tau1=0.1, tau2=0.1;
		double Fi,CRi;

		Fi=m_inds[parent]->GetWeightF();
		F=Fi;//inheritance from parent (i)
		CRi=m_inds[parent]->GetCrossProbCR();
		CR=CRi;//inheritance from parent (i)

		double rand2=(double) rand() / RAND_MAX;
		if (rand2 < tau1) {
			double rand1=(double) rand() / RAND_MAX;
			F=Fl+rand1*Fu;
		}

		double rand4=(double) rand() / RAND_MAX;
		if (rand4 < tau2) {
			double rand3=(double) rand() / RAND_MAX;
			CR=rand3;
		}
		repairF_CRNumber(&F, m_params->minF, m_params->maxF);
		repairF_CRNumber(&CR, m_params->minCR, m_params->maxCR);
	} else if (m_params->strategyDE==121) {//from jDE (Brest et al.) => DEMOwSA//jDE-strat
		const double Fl=0.1,Fu=0.8;
		const double CRl=0.1,CRu=0.4;
		const double tau1=0.1, tau2=0.1;
		double Fi,CRi;

		bool adapt=1;
		if (adapt) {
			//inheritance from PARENT
			vector<double>& FvectParent = m_inds[parent]->GetWeightFVect();
			vector<double>& CRvectParent = m_inds[parent]->GetCrossProbCRVect();
			vector<double>& Fvect=candidate->GetWeightFVect()=FvectParent;
			vector<double>& CRvect=candidate->GetCrossProbCRVect()=CRvectParent;

			//reserve some space
			int numberOfStrategies=4;

			while (Fvect.size()<numberOfStrategies) {
					double Finit=0.6;
					double CRinit=0.4;
					Fvect.push_back(Finit);
					CRvect.push_back(CRinit);
			}

			do {
				selDESubStrategy = rand()%numberOfStrategies;
			} while (selDESubStrategy == 1 || selDESubStrategy == 2);

			double sampleF=Fvect[selDESubStrategy];
			double sampleCR=CRvect[selDESubStrategy];

			F=sampleF;
			CR=sampleCR;

			double rand2=(double) rand() / RAND_MAX;
			if (rand2 < tau1) {
				double rand1=(double) rand() / RAND_MAX;
				F=Fl+rand1*Fu;
			}

			double rand4=(double) rand() / RAND_MAX;
			if (rand4 < tau2) {
				double rand3=(double) rand() / RAND_MAX;
				//CR=rand3;
				CR = CRl + rand3*CRu;
			}

			//TODO: commented out, why?
			//repairF_CRNumber(&F, m_params->minF, m_params->maxF);
			//repairF_CRNumber(&CR, m_params->minCR, m_params->maxCR);

			//copy+write back the F?
			Fvect[selDESubStrategy]=F;
			CRvect[selDESubStrategy]=CR;
		}
	}

	i_rand = random(m_probl->GetNumVar());
	i = i_rand;
	for (UINT k = 0; k < m_probl->GetNumVar(); k++) {
		if (selDESubStrategy==3)
		{
			// tmp[n] = tmp[n] + (mRandom())*((*pold)[r1][n]-tmp[n]) + F*((*pold)[r2][n]-(*pold)[r3][n]);
			double rand1=(double) rand() / RAND_MAX;
			result.at(i) =
				m_inds[parent]->GetGene(i) +
				//m_params->GetWeight() *
				rand1 *
				(m_inds.at(i1)->GetGene(i) - m_inds[parent]->GetGene(i)) +
				F *
				(m_inds.at(i2)->GetGene(i) - m_inds.at(i3)->GetGene(i));

		} else
		//patch by AlesZ: increased CR accuracy
// 		if (random(1000000) < (1000000 * m_params->GetCrossProb()) || (i == i_rand))
		if (random(1000000) < (1000000 * CR) || (i == i_rand)) {
			if (m_params->strategyDE==121) {
				if (selDESubStrategy == 0 ) {
					// s20
					//v_{i,G} = F * (x_{i1,i,G} - x_{i2,i,G}) + x_{i3,i,G}
					result.at(i) =
						//m_params->GetWeight() *
						F *
						(m_inds.at(i1)->GetGene(i) - m_inds.at(i2)->GetGene(i)) +
						m_inds.at(i3)->GetGene(i);
				} else if (selDESubStrategy == 1 ) {
					// s23: JANDE/rand-to-best/1/bin
					// tmp[n] = tmp[n] + F*(bestit[n] - tmp[n]) + F*((*pold)[r1][n]-(*pold)[r2][n]);
					result.at(i) =
						m_inds[parent]->GetGene(i) +
						//m_params->GetWeight() *
						F *
						(bestit->GetGene(i) - m_inds[parent]->GetGene(i)) +
										F *
						(m_inds.at(i1)->GetGene(i) - m_inds.at(i2)->GetGene(i));
				} else if (selDESubStrategy == 2 ) {
					// s29: JANDE/best/2/bin
					// tmp[n] = bestit[n] +
					//        ((*pold)[r1][n]+(*pold)[r2][n]-(*pold)[r3][n]-(*pold)[r4][n])*F;
					result.at(i) =
						bestit->GetGene(i) +
						F *
						(m_inds.at(i1)->GetGene(i) - m_inds.at(i2)->GetGene(i) +
						m_inds.at(i3)->GetGene(i) - m_inds.at(i4)->GetGene(i));
				} else exit(1); //error in DEMO
            } else if (m_params->strategyDE==14) {//ND-rand/1/bin
				//v_{i,G} = F * (bestit_{i,G} - x_{i2,i,G}) + x_{i3,i,G}
				result.at(i) =
					F *
					(m_inds.at(i1)->GetGene(i) - m_inds.at(i2)->GetGene(i)) +
					bestit->GetGene(i);
			} else if (m_params->strategyDE==15) {//ND-rand/1/bin
				//cur-to-rand
				//v_{i,G} = F * (bestit_{i,G} - x_{i2,i,G}) + x_{i3,i,G}
				result.at(i) =
					F *
					(m_inds.at(i1)->GetGene(i) - m_inds.at(i2)->GetGene(i)) +
					bestit->GetGene(i);
			} else
				//v_{i,G} = F * (x_{i1,i,G} - x_{i2,i,G}) + x_{i3,i,G}
				result.at(i) =
					//m_params->GetWeight() *
					F *
					(m_inds.at(i1)->GetGene(i) - m_inds.at(i2)->GetGene(i)) +
					m_inds.at(i3)->GetGene(i);
		} else
			result.at(i) = m_inds.at(parent)->GetGene(i);
		i = (i + 1) % m_probl->GetNumVar();
	}
	return m_probl->GetRoundVector(result);
}
//---------------------------------------------------------------------------

vector<UINT> DemoPop::SortPop()
{
	vector<UINT> result;
	result.clear();
	return result;
}
//---------------------------------------------------------------------------

vector<UINT> DemoPop::SortPopRandom()
{
	UINT i;
	vector<double> rand_num;
	rand_num.clear();

	for (i = 0; i < m_inds.size(); i++)
		rand_num.push_back((double) rand() / RAND_MAX);
	return GetSorted(rand_num);
}
//---------------------------------------------------------------------------

void DemoPop::TruncPop(vector<UINT> &sort_vect)
{
	UINT i;
	UINT size = sort_vect.size();
	vector<DemoInd*> inds;
	inds.clear();

	for (i = 0; i < size; i++) {
		DemoInd *ind = new DemoInd();
		*ind = *m_inds.at(sort_vect.at(i));//time consuming: full deep copy!!
		inds.push_back(ind);
	}

	for (i = 0; i < size; i++)
		*m_inds.at(i) = *inds.at(i);
	while (m_inds.size() > size) {
		delete m_inds.at(m_inds.size() - 1);
		m_inds.pop_back();
	}
	for (i = 0; i < size; i++)
		delete inds.at(i);
}
//---------------------------------------------------------------------------

void DemoPop::AppendInd(DemoInd *ind)
{
	m_inds.push_back(ind);
}
//---------------------------------------------------------------------------

string DemoPop::GetPopString()
{
	UINT i;
	stringstream result;

	for (i = 0; i < m_inds.size(); i++)
		result << m_inds.at(i)->GetIndString();

	return result.str();
}
//---------------------------------------------------------------------------

string DemoPop::GetPopCritString()
{
	UINT i;
	stringstream result;

	for (i = 0; i < m_inds.size(); i++)
		result << m_inds.at(i)->GetIndCritString();

	return result.str();
}
//---------------------------------------------------------------------------

string DemoPop::GetFrontString(bool front_long)
{
	stringstream result;
	UINT i, j, dominate;

	vector<UINT> n;
	n.clear();

	for (i = 0; i < m_inds.size(); i++) {
		n.push_back(0);
		for (j = 0; j < m_inds.size(); j++) {
			if (i == j) continue;
			dominate = m_probl->GetDomination(m_inds.at(i), m_inds.at(j));
			if (dominate == g_domin_second)
				n.at(i)++;
		}
#define OUTPUTALL 1
		if (n.at(i) == 0|| OUTPUTALL) {
			if (front_long)
				result << m_inds.at(i)->GetIndString();
			else
				result << m_inds.at(i)->GetIndCritString();
		}
	}

	return result.str();
}
//---------------------------------------------------------------------------

void DemoPop::swapInds(const vector<DemoInd*> &m_indsIn, vector<DemoInd*> &m_indsOut) {
	//swap curr inds, for Prunning strategy nr. 4
	m_indsOut = m_inds;
	m_inds = m_indsIn;
}

void DemoPop::returnInds(const vector<DemoInd*> &m_indsIn) {
	//hey, go for it and make a lazy naive implementation!
	m_inds = m_indsIn;
}
