/*===========================================================================

	Copyright  2007 Tea Tuar

	This material is provided "as is", with no warranty expressed or implied.
	Any use is at your own risk. Permission to use or copy this software for
	any purpose is hereby granted without fee, provided this notice is
	retained on all copies. Permission to modify the code and to distribute
	modified code is granted, provided a notice that the code was modified is
	included with the above copyright notice.

	http://dis.ijs.si/tea/research.htm

	===========================================================================

	Header file.

	Defines the class DemoPop.

	file: demo_pop.h
	author: Tea Tusar (tea.tusar@ijs.si)
	created: 17.02.2006
	last change: $date$

	===========================================================================
*/

#ifndef demo_popH
#define demo_popH
//---------------------------------------------------------------------------

#include "demo_ind.h"
#include "demo_params.h"
#include "probl.h"
#include "file_stat.h"
#include <gsl/gsl_rng.h>//added by AlesZ
//---------------------------------------------------------------------------

class FileStat;
//---------------------------------------------------------------------------

class DemoPop
{
protected:
	/** Vector of individuals. */
	vector<DemoInd*> m_inds;

	/** Parameters. */
	DemoParams *m_params;

	/** Problem. */
	Probl *m_probl;

	/** Normally distributed random number generator*/
	static gsl_rng *rNormalDistr;

public:
	inline vector<DemoInd*>& GetInds() {return m_inds;};
	inline DemoInd* GetInd(UINT index) {return m_inds.at(index);};
	inline Probl* GetProbl() {return m_probl;};

	/** Class constructor. */
	DemoPop();

	/** Class destructor. */
	virtual ~DemoPop();

	/** Initialization function.
	params: params paramters
	params: probl problem
	*/
	void InitPop(DemoParams *params, Probl *probl);

	/** Function fills the first population with random individuals. */
	void CreateFirstPop();

	/** Function evaluates all the individual in the population.
	params: files files for output information
	*/
	void EvaluatePop(FileStat *files);

	/** Function calculates and returns a new candidate.
	params: parent the parent of the candidate
	returns: the genes of the candidate
	*/
	vector<double> GetCandidate(UINT parent, double &F, double &CR, DemoInd *candidate, const vector<int> &nondominatedInd);

	/** Virtual function for sorting the population - to be overrided by the
	descendent classes. */
	virtual vector<UINT> SortPop();

	/** Function randomly sorts the population.
	returns: randomly sorted indexes of the individuals
	*/
	vector<UINT> SortPopRandom();

	/** Truncation function for the whole population truncates the population
	to contain only the individuals from the sort_vect.
	params: sort_vect vector with indexes of the individuals to be left in the
	population
	*/
	void TruncPop(vector<UINT> &sort_vect);

	/** Function appends the individual to the last spot in the population.
	params: ind the current individual
	*/
	void AppendInd(DemoInd *ind);

	/** Function constructs and returns the string of the population using the
	strings of the individuals.
	returns: the string describing the population
	*/
	string GetPopString();

	/** Function constructs and returns the string of the population using the
	strings of the individuals. The string contains only the information on the
	criteria.
	returns: the string describing the population
	*/
	string GetPopCritString();

	/** Function constructs and returns the string of the current front using the
	strings of the individuals.
	params: front_long determines whether all the information should be written
	returns: the string describing the front
	*/
	string GetFrontString(bool front_long);

	/** HACK: reuse this object for strategy-4 prunning: copy them all and take a long time*/
	void swapInds(const vector<DemoInd*> &m_indsIn, vector<DemoInd*> &m_indsOut);

	/** HACK: Copy once again them all in */
	void returnInds(const vector<DemoInd*> &m_indsIn);
};
//---------------------------------------------------------------------------
#endif


