/*===========================================================================

	Copyright  2007 Tea Tuar

	This material is provided "as is", with no warranty expressed or implied.
	Any use is at your own risk. Permission to use or copy this software for
	any purpose is hereby granted without fee, provided this notice is
	retained on all copies. Permission to modify the code and to distribute
	modified code is granted, provided a notice that the code was modified is
	included with the above copyright notice.

	http://dis.ijs.si/tea/research.htm

	===========================================================================

	Implementation file.

	file: demo_pop_ibea.cpp
	author: Tea Tusar (tea.tusar@ijs.si)
	created: 18.02.2006
	last change: $date$

	===========================================================================
*/

#include "demo_pop_ibea.h"
//---------------------------------------------------------------------------

const UINT g_ibea_eps = 0;
const UINT g_ibea_hyp = 1;
//---------------------------------------------------------------------------

DemoPopIbea::DemoPopIbea()
	: DemoPop()
{
	m_indicator = g_ibea_eps;
	m_rho = 2;
	m_kappa = 0.05;

	m_fitness.clear();
	m_bounds.clear();
}
//---------------------------------------------------------------------------

DemoPopIbea::~DemoPopIbea()
{
}
//---------------------------------------------------------------------------

vector<UINT> DemoPopIbea::SortPop()
{
	vector<UINT> result;
	result.clear();
	UINT i, j;
	vector<UINT>::iterator k, worst;

	if (m_inds.size() == m_params->GetPopSize()) {
		for (i = 0; i < m_inds.size(); i++)
			result.push_back(i);
		return result;
	}

	CalcBounds();
	CalcFitness();

	vector<UINT> remaining;
	for (j = 0; j < m_inds.size(); j++) {
		remaining.push_back(j);
	}

	for (i = m_inds.size(); i > m_params->GetPopSize(); i--) {
		worst = remaining.begin();
		for (k = remaining.begin(); k != remaining.end(); k++)
			if (m_fitness.at(*worst) < m_fitness.at(*k)) {
				worst = k;
			}
		for (k = remaining.begin(); k != remaining.end(); k++)
			m_fitness.at(*k) -=	m_fitness_comp.at(*worst).at(*k);
		remaining.erase(worst);
	}

	for (j = 0; j < remaining.size(); j++)
		result.push_back(remaining.at(j));

	return result;
}
//---------------------------------------------------------------------------

void DemoPopIbea::CalcBounds()
{
	Bound b;
	m_bounds.clear();
	for (UINT i = 0; i < m_probl->GetNumFunc(); i++) {
		m_bounds.push_back(b);
		m_bounds.at(i).m_min = m_inds.at(0)->GetCriterion(i);
		m_bounds.at(i).m_max = m_inds.at(0)->GetCriterion(i);
	}
	for (UINT j = 0; j < m_inds.size(); j++) {
		for (UINT i = 0; i < m_probl->GetNumFunc(); i++) {
			if (m_bounds.at(i).m_min > m_inds.at(j)->GetCriterion(i))
				m_bounds.at(i).m_min = m_inds.at(j)->GetCriterion(i);
			if (m_bounds.at(i).m_max < m_inds.at(j)->GetCriterion(i))
				m_bounds.at(i).m_max = m_inds.at(j)->GetCriterion(i);
		}
	}

	for (UINT i = 0; i < m_probl->GetNumFunc(); i++) {
		if (m_bounds.at(i).m_min ==	m_bounds.at(i).m_max)
		// All individuals have the same value of objective i:
			m_bounds.at(i).m_max = m_bounds.at(i).m_min + 1;
	}
}
//---------------------------------------------------------------------------

void DemoPopIbea::CalcFitness()
{
	UINT i, j;
	double max_abs_ind = 0;
	vector<double> tmp;

	m_fitness_comp.clear();
	for (i = 0; i < m_inds.size(); i++) {
		tmp.clear();
		for (j = 0; j < m_inds.size(); j++) {
			tmp.push_back(CalcIndicator(m_inds.at(i), m_inds.at(j)));
			if (max_abs_ind < fabs(tmp.at(j))) {
				max_abs_ind = fabs(tmp.at(j));
			}
		}
		m_fitness_comp.push_back(tmp);
	}

	for (i = 0; i < m_inds.size(); i++)
		for (j = 0; j < m_inds.size(); j++) {
			if (max_abs_ind == 0) {
				m_fitness_comp.at(i).at(j) = 0;
			}
			else {
				m_fitness_comp.at(i).at(j) =
					exp(-m_fitness_comp.at(i).at(j) / max_abs_ind / m_kappa);
			}
		}

	m_fitness.clear();
	for (i = 0; i < m_inds.size(); i++) {
		m_fitness.push_back(0);
		for (j = 0; j < m_inds.size(); j++) {
			if (i != j)
				m_fitness.at(i) += m_fitness_comp.at(j).at(i);
		}
	}
}
//---------------------------------------------------------------------------

double DemoPopIbea::CalcIndicator(DemoInd *ind1, DemoInd *ind2)
{
	double result=1000;//patch by AlesZ

	if (m_indicator == g_ibea_eps)
		result = CalcEps(ind1, ind2);
	else if (m_indicator == g_ibea_hyp) {
		if (m_probl->GetDomination(ind1, ind2) == g_domin_first)
			result = - CalcHyp(ind1, ind2, m_probl->GetNumFunc());
		else
			result = CalcHyp(ind2, ind1, m_probl->GetNumFunc());
	}

	return result;
}
//---------------------------------------------------------------------------

double DemoPopIbea::CalcEps(DemoInd *ind1, DemoInd *ind2)
{
	double r, tmp;
	double result;

	r = m_bounds.at(0).m_max - m_bounds.at(0).m_min;
	result = (ind1->GetCriterion(0) - m_bounds.at(0).m_min) / r -
		(ind2->GetCriterion(0) - m_bounds.at(0).m_min) / r;

	for (UINT i = 1; i < m_probl->GetNumFunc(); i++) {
		r = m_bounds.at(i).m_max - m_bounds.at(i).m_min;
		tmp = (ind1->GetCriterion(i) - m_bounds.at(i).m_min) / r -
		(ind2->GetCriterion(i) - m_bounds.at(i).m_min) / r;
		if (tmp > result)
			result = tmp;
	}
	return result;
}
//---------------------------------------------------------------------------

double DemoPopIbea::CalcHyp(DemoInd *ind1, DemoInd *ind2, UINT dim)
{
	double f1, f2, r, max;
	double result;

	r = m_rho * (m_bounds.at(dim - 1).m_max - m_bounds.at(dim - 1).m_min);
	max = m_bounds.at(dim - 1).m_min + r;

	f1 = ind1->GetCriterion(dim - 1);

	if (ind2 == NULL)
		f2 = max;
	else
		f2 = ind2->GetCriterion(dim - 1);

	if (dim == 1) {
		if (f1 < f2)
			result = (f2 - f1) / r;
		else
			result = 0;
	}
	else {
		if (f1 < f2) {
			result = CalcHyp(ind1, NULL, dim - 1) *	(f2 - f1) / r;
			result += CalcHyp(ind1, ind2, dim - 1) * (max - f2) / r;
		}
		else {
			result = CalcHyp(ind1, ind2, dim - 1) * (max - f2) / r;
		}
	}

	return result;
}
//---------------------------------------------------------------------------

