/*===========================================================================

	Copyright  2007 Tea Tuar

	This material is provided "as is", with no warranty expressed or implied.
	Any use is at your own risk. Permission to use or copy this software for
	any purpose is hereby granted without fee, provided this notice is
	retained on all copies. Permission to modify the code and to distribute
	modified code is granted, provided a notice that the code was modified is
	included with the above copyright notice.

	http://dis.ijs.si/tea/research.htm

	===========================================================================

	Header file.

	Defines the class DemoPopIbea, which is derived from the class DemoPop.

	file: demo_pop_ibea.h
	author: Tea Tusar (tea.tusar@ijs.si)
	created: 18.02.2006
	last change: $date$

	===========================================================================
*/

#ifndef demo_pop_ibeaH
#define demo_pop_ibeaH
//---------------------------------------------------------------------------

#include "demo_ind.h"
#include "demo_pop.h"
//---------------------------------------------------------------------------

	/** Indicator types. */
extern const UINT g_ibea_eps;
extern const UINT g_ibea_hyp;
//---------------------------------------------------------------------------

class DemoPopIbea : public DemoPop
{
	/** Chosen indicator. */
	UINT m_indicator;

	/** IBEA's fitness. */
	vector<double> m_fitness;

	/** Vector of IBEA's fitness components. */
	vector< vector<double> > m_fitness_comp;

	/** IBEA's rho. */
	double m_rho;

	/** IBEA's kappa. */
	double m_kappa;

	struct Bound
	{
		double m_min;
		double m_max;
	};
	/** Vector of criteria bounds. */
	vector<Bound> m_bounds;

public:
	inline void SetIndicator(UINT indicator) {m_indicator = indicator;};
	inline void SetRho(double rho) {m_rho = rho;};
	inline void SetKappa(double kappa) {m_kappa = kappa;};

	inline UINT GetIndicator() {return m_indicator;};
	inline double GetRho() {return m_rho;};
	inline double GetKappa() {return m_kappa;};

	/** Class constructor. */
	DemoPopIbea();

	/** Class destructor. */
	~DemoPopIbea();

	/** Function sorts the individuals regarding their fitness.
	returns: indexes of the best m_pop_size individuals
	*/
	vector<UINT> SortPop();

	/** Function calculates the criteria bounds and stores them in m_bounds. */
	void CalcBounds();

	/** Function calculates the fitness of all individuals in the population
	with regard to the given indicator. */
	void CalcFitness();

	/** Function calculates and returns the indicator of the two individuals.
	params: ind1 the first individual
	params: ind2 the second individual
	returns: value of the indicator
	*/
	double CalcIndicator(DemoInd *ind1, DemoInd *ind2);

	/** Function calculates and returns the epsilon indicator of the two
	individuals.
	params: ind1 the first individual
	params: ind2 the second individual
	returns: value of the epsilon indicator
	*/
	double CalcEps(DemoInd *ind1, DemoInd *ind2);

	/** Function calculates and returns the hypervolume indicator of the two
	individuals.
	params: ind1 the first individual
	params: ind2 the second individual
	returns: value of the hypervolume indicator
	*/
	double CalcHyp(DemoInd *ind1, DemoInd *ind2, UINT dim);
};
//---------------------------------------------------------------------------
#endif


