/*===========================================================================

	Copyright  2007 Tea Tuar

	This material is provided "as is", with no warranty expressed or implied.
	Any use is at your own risk. Permission to use or copy this software for
	any purpose is hereby granted without fee, provided this notice is
	retained on all copies. Permission to modify the code and to distribute
	modified code is granted, provided a notice that the code was modified is
	included with the above copyright notice.

	http://dis.ijs.si/tea/research.htm

	===========================================================================

	Header file.

	Defines the class DemoPopNsga, which is derived from DemoPop.

	file: demo_pop_nsga.h
	author: Tea Tusar (tea.tusar@ijs.si)
	created: 18.02.2006
	last change: $date$

	===========================================================================
*/

#ifndef demo_pop_nsgaH
#define demo_pop_nsgaH
//---------------------------------------------------------------------------

#include "demo_ind.h"
#include "demo_pop.h"
#include "demo_utilities.h"
//---------------------------------------------------------------------------

class DemoPopNsga : public DemoPop
{
	/** Whether to use controlled elitism in fronts. */
	bool m_elitism;

	/** The controlled elitism factor. */
	double m_elitism_r;

	/** Vector of crowding distances of all individuals. */
	vector<double> m_crow_dist;

	/** Vector of ranks of all individuals. */
	vector<UINT> m_ranks;

	/** Vector of fronts. */
	vector< vector<UINT> > m_fronts;

public:
	inline void SetElitism(bool elitism) {m_elitism = elitism;};
	inline void SetElitismR(double elitism_r) {m_elitism_r = elitism_r;};

	inline UINT GetElitism() {return m_elitism;};
	inline double GetElitismR() {return m_elitism_r;};

	/** Class constructor. */
	DemoPopNsga();

	/** Class destructor. */
	~DemoPopNsga();

	/** Function sorts the indviduals.
	returns: indexes of the best m_pop_size individuals
	*/
	vector<UINT> SortPop();

	/** Function sorts the individuals with regard to their position in fronts
	and their crowding distance value. Does not use controlled elitism!
	returns: indexes of the best m_pop_size individuals
	*/
	vector<UINT> SortPopNsga();

	/** Function sorts the individuals with regard to their position in fronts
	and their crowding distance value. Uses controlled elitism!
	returns: indexes of the best m_pop_size individuals
	*/
	vector<UINT> SortPopNsgaElitism();

	/** Function calculates the fronts and stores them in m_fronts. */
	void CalcFronts();

	/** Function calculates the crowding distances of the front_num-th front.
	params: front_num number of front for which the crowding distances are
	calculated
	*/
	void CalcCrowDist(UINT front_num);
	
private:
	void RestoreCrowdDistVals(vector<UINT> &front, double &copy_sum, vector< vector<UINT> > &copies, vector<UINT> &tmp_copy);
	void oneFor(UINT &i, UINT &j, UINT&k, vector<UINT> &tmp_copy, vector< vector<UINT> > &copies);
};
//---------------------------------------------------------------------------
#endif


