/*===========================================================================

	Copyright  2007 Tea Tuar

	This material is provided "as is", with no warranty expressed or implied.
	Any use is at your own risk. Permission to use or copy this software for
	any purpose is hereby granted without fee, provided this notice is
	retained on all copies. Permission to modify the code and to distribute
	modified code is granted, provided a notice that the code was modified is
	included with the above copyright notice.

	http://dis.ijs.si/tea/research.htm

	===========================================================================

	Implementation file.

	file: demo_pop_spea.cpp
	author: Tea Tusar (tea.tusar@ijs.si)
	created: 09.03.2006
	last change: $date$

	===========================================================================
*/

#include "demo_pop_spea.h"
//---------------------------------------------------------------------------

DemoPopSpea::DemoPopSpea()
	: DemoPop()
{
}
//---------------------------------------------------------------------------

DemoPopSpea::~DemoPopSpea()
{
}
//---------------------------------------------------------------------------

vector<UINT> DemoPopSpea::SortPop()
{
	UINT i;
	UINT pop_size = m_params->GetPopSize();
	UINT num_inds = m_inds.size();
	const UINT k = (UINT)sqrt(num_inds);

	vector<UINT> result;
	result.clear();

	if (num_inds == pop_size) {
		for (i = 0; i < pop_size; i++)
			result.push_back(i);
	}

	else {

		CalcRawFitness();
		CalcDistance();

		m_fitness.clear();
		for (i = 0; i < num_inds; i++) {
			m_fitness.push_back(m_raw_fitness.at(i) + 1.0 / (2 + m_distance.at(i).at(k)));
		}

		vector<UINT> sorted;
		sorted = GetSorted(m_fitness);

		if (m_fitness.at(sorted.at(pop_size)) < 1) {
			UINT last = pop_size;
			while ((last < num_inds - 1) && (m_fitness.at(sorted.at(last + 1)) < 1))
				last++;
			for (i = 0; i <= last; i++)
				result.push_back(sorted.at(i));
			for (i = last + 1; i < num_inds; i++)
				RemoveFromRecord(sorted.at(i));

			result = TruncatePop(result, last - pop_size + 1);
		}

		else {
			for (i = 0; i < pop_size; i++)
				result.push_back(sorted.at(i));
		}

	}

	return result;
}
//---------------------------------------------------------------------------

void DemoPopSpea::CalcRawFitness()
{
	UINT i, j;
	vector<UINT> strength;
	strength.clear();
	for (i = 0; i < m_inds.size(); i++)
		strength.push_back(0);

	for (i = 0; i < m_inds.size(); i++)
		for (j = 0; j < m_inds.size(); j++)
			if (m_probl->GetDomination(m_inds.at(i), m_inds.at(j)) == g_domin_first)
			{
				strength.at(i)++;
			}

	m_raw_fitness.clear();
	for (i = 0; i < m_inds.size(); i++) {
		m_raw_fitness.push_back(0);
		for (j = 0; j < m_inds.size(); j++)
			if (m_probl->GetDomination(m_inds.at(j), m_inds.at(i)) == g_domin_first) {
				m_raw_fitness.at(i) += strength.at(j);
			}
	}
}
//---------------------------------------------------------------------------

void DemoPopSpea::CalcDistance()
{
	UINT i, j;

	m_distance.clear();
	m_copies.clear();
	m_nn.clear();

	vector<int> tmp1(m_inds.size(), -1);
	for (i = 0; i < m_inds.size(); i++) {
		m_copies.push_back(1);
		m_nn.push_back(tmp1);
	}

	vector<double> tmp2(m_inds.size(), 0);
	for (i = 0; i < m_inds.size(); i++) {
		m_nn.at(i).at(0) = i;
		for (j = 0; j < i; j++) {
			tmp2.at(j) = m_distance.at(j).at(i);
		}
		tmp2.at(i) = 0;
		for (j = i + 1; j < m_inds.size(); j++) {
			tmp2.at(j) = m_probl->GetDistance(m_inds.at(i)->GetCriteria(),
																				m_inds.at(j)->GetCriteria());
			if (tmp2.at(j) == 0) {
				m_nn.at(i).at(m_copies.at(i)) = j;
				m_nn.at(j).at(m_copies.at(j)) = i;
				m_copies.at(i)++;
				m_copies.at(j)++;
			}
		}
		m_distance.push_back(tmp2);
	}
}
//---------------------------------------------------------------------------

void DemoPopSpea::RemoveFromRecord(UINT index)
{
	for (UINT i = 0; i < m_inds.size(); i++)	{
		if ((m_distance.at(index).at(i) == 0) && (m_copies.at(i) > 0))
			m_copies.at(i)--;
	}
	m_copies.at(index) = 0;
}
//---------------------------------------------------------------------------

vector<UINT> DemoPopSpea::TruncatePop(vector<UINT> &indexes,
																			UINT num_redundant)
{
	for (UINT j = 0; j < num_redundant; j++) {

		UINT i;
		vector<UINT> marked(indexes.size(), 0);
		UINT max_copies = 0;
		UINT count = 0;

		for (i = 0; i < m_inds.size(); i++) {
			if (m_copies.at(i) > max_copies) {
				count = 0;
				max_copies = m_copies.at(i);
			}
			if (m_copies.at(i) == max_copies)	{
				marked.at(count) = i;
				count++;
			}
		}

		if (count > max_copies) {
			vector<int> neighbor(count, 1);
			while (count > max_copies) {
				double min_dist = MAXINT;
				int count2 = 0;

				for (i = 0; i < count; i++)	{
					double my_dist = -1;

					while (my_dist == -1 && neighbor.at(i) < (int)m_inds.size()) {
						my_dist = GetDistanceToNN(marked.at(i), neighbor.at(i));
						neighbor.at(i)++;
					}

					if (my_dist < min_dist) {
						count2 = 0;
						min_dist = my_dist;
					}

					if (my_dist == min_dist) {
						marked.at(count2) = marked.at(i);
						neighbor.at(count2) = neighbor.at(i);
						count2++;
					}
				}
				count = count2;
				if (min_dist == -1)
					break;
			}
		}

		UINT delete_index = marked.at(random(count));
		RemoveFromRecord(delete_index);
		vector<UINT>::iterator k;
		for (k = indexes.begin(); k < indexes.end(); k++)
			if (*k == delete_index) {
				indexes.erase(k);
				break;
			}
	}

	return indexes;
}
//---------------------------------------------------------------------------

int DemoPopSpea::GetNN(UINT index, UINT k)
{
	if (m_nn.at(index).at(k) == -1) {

		double min_dist = MAXINT;
		int min_index = -1;
		int prev_min_index = m_nn.at(index).at(k - 1);
		double prev_min_dist = m_distance.at(index).at(prev_min_index);

		for (UINT i = 0; i < m_inds.size(); i++) {
			double my_dist = m_distance.at(index).at(i);

			if (my_dist < min_dist && index != i)	{
				if (my_dist > prev_min_dist ||
					 ((my_dist == prev_min_dist) && ((int)i > prev_min_index))) {
					min_dist = my_dist;
					min_index = i;
				}
			}
		}

		m_nn.at(index).at(k) = min_index;
	}

	return m_nn.at(index).at(k);
}
//---------------------------------------------------------------------------

double DemoPopSpea::GetDistanceToNN(UINT index, UINT k)
{
	double result = -1;

	int neighbor_index = GetNN(index, k);

	if (m_copies.at(neighbor_index) != 0)
		result = m_distance.at(index).at(neighbor_index);

	return result;
}
//---------------------------------------------------------------------------

