/*===========================================================================

	Copyright  2007 Tea Tuar

	This material is provided "as is", with no warranty expressed or implied.
	Any use is at your own risk. Permission to use or copy this software for
	any purpose is hereby granted without fee, provided this notice is
	retained on all copies. Permission to modify the code and to distribute
	modified code is granted, provided a notice that the code was modified is
	included with the above copyright notice.

	http://dis.ijs.si/tea/research.htm

	===========================================================================

	Header file.

	Defines the class DemoPopSpea, which is derived from the class DemoPop.

	file: demo_pop_spea.h
	author: Tea Tusar (tea.tusar@ijs.si)
	created: 09.03.2006
	last change: $date$

	===========================================================================
*/

#ifndef demo_pop_speaH
#define demo_pop_speaH
//---------------------------------------------------------------------------

#include "demo_ind.h"
#include "demo_pop.h"
#include "demo_utilities.h"
//---------------------------------------------------------------------------

class DemoPopSpea : public DemoPop
{
	/** SPEA2's fitness. */
	vector<double> m_fitness;

	/** SPEA2's raw fitness. */
	vector<double> m_raw_fitness;

	/** Distances between individuals. */
	vector< vector<double> > m_distance;

	/** Copies of individuals. */
	vector<UINT> m_copies;

	/** Vector of nearest neighbors for all individuals. */
	vector< vector<int> > m_nn;

public:

	/** Class consturctor. */
	DemoPopSpea();

	/** Class destructor. */
	~DemoPopSpea();

	/** Function sorts the individuals regarding their fitness.
	returns: indexes of the best m_pop_size individuals
	*/
	vector<UINT> SortPop();

	/** Function calculates the raw fitness of all individuals and stores it to
	m_raw_fitness. */
	void CalcRawFitness();

	/** Function calculates the distances between all individuals and stores them
	to m_distance. */
	void CalcDistance();

	/** Function removes the individual with index from the record of copies.
	params: index index of the individual to be removed from the record
	*/
	void RemoveFromRecord(UINT index);

	/** Function truncates the population with individuals given by indexes
	until num_redundant individuals are removed.
	params: indexes indexes of the individuals
	params: num_redundant number of individuals to be truncated
	returns: indexes of the remaining individuals
	*/
	vector<UINT> TruncatePop(vector<UINT> &indexes,
													 UINT num_redundant);

	/** Function calculates and returns the k-th nearest neighbor of the index-th
	individual.
	params: index index of the individual
	params: k
	returns: the index of the individual's k-th nearest neighbor
	*/
	int GetNN(UINT index, UINT k);

	/** Function calculates and returns the distance to the k-th nearest neighbor
	of the index-th individual.
	params: index index of the individual
	params: k the neighbor to which the distance is calculated
	returns: the distance between individual index and its k-th nearest neighbor
	*/
	double GetDistanceToNN(UINT index, UINT k);
};
//---------------------------------------------------------------------------
#endif


