/***************************************************************************
 *   Copyright (C) 2007 by Aleš Zamuda, Tea Tušar (IJS)   *
 *   ales.zamuda@uni-mb.si, tea.tusar@ijs.si   *
 *                                                                         *
 *   Permission is hereby granted, free of charge, to any person obtaining *
 *   a copy of this software and associated documentation files (the       *
 *   "Software"), to deal in the Software without restriction, including   *
 *   without limitation the rights to use, copy, modify, merge, publish,   *
 *   distribute, sublicense, and/or sell copies of the Software, and to    *
 *   permit persons to whom the Software is furnished to do so, subject to *
 *   the following conditions:                                             *
 *                                                                         *
 *   The above copyright notice and this permission notice shall be        *
 *   included in all copies or substantial portions of the Software.       *
 *                                                                         *
 *   THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,       *
 *   EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF    *
 *   MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. *
 *   IN NO EVENT SHALL THE AUTHORS BE LIABLE FOR ANY CLAIM, DAMAGES OR     *
 *   OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, *
 *   ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR *
 *   OTHER DEALINGS IN THE SOFTWARE.                                       *
 ***************************************************************************/
#include "demowsa_localsearch.h"

#include "demo_ind.h"
#include "probl.h"

static DemoInd myStaticLastInd;
static std::vector<double> lastEps;
static bool firstCallToEval;
// static double fval;

void ensureEvaluated(void *cd, double *x, int nparam) {
	Probl *probl = (Probl*)cd;
	static bool initialized=false;
	if (!initialized) {
		std::vector<double> initGenes;
		initGenes.resize(nparam);
		myStaticLastInd.SetGenes(initGenes);

		std::vector<double> initCriteria;
		initCriteria.resize(probl->GetNumFunc());
		myStaticLastInd.SetCriteria(initCriteria);
		initialized=true;

		lastEps.resize(probl->GetNumFunc());
	}

	bool isSame=true;
	for (int i = 0; i < nparam; i++) {
		if (x[i]!=myStaticLastInd.GetGene(i)) {
			isSame=false;
			myStaticLastInd.SetGene(i, x[i]);	//transcript x_i
		}
	}

	if (!isSame) {
		myStaticLastInd.Evaluate(probl);


		Probl *m_probl=probl;
		DemoInd &candidate=myStaticLastInd;

// 		for (int i = 0; i < m_probl->GetNumVar(); i++) {
// 			double g = candidate.GetGene(i);
// 			double L = m_probl->GetVarLower(i);
// 			double U = m_probl->GetVarUpper(i);
// 			if (g < L || g > U) {
// 				cout << "EEEx g=" << g << ", L=" << L << ", U="<< U<<endl;
// 			}
// 		}
//
// 		for (int i = 0; i < m_probl->GetNumFunc(); i++) {
// 			double c = candidate.GetCriterion(i);
// 			double L = m_probl->GetFuncLower(i);
// 			double U = m_probl->GetFuncUpper(i);
// 			if (c < L /*|| c > U*/) {
// 				cout << "EEEf c=" << c << ", L=" << L << ", U="<< U<<endl;
// 			}
// 		}

		if (firstCallToEval) {
			firstCallToEval=false;
			for (int i = 0; i < probl->GetNumFunc(); i++)
				lastEps[i]=myStaticLastInd.GetCriterion( i );
		}

/*		fval=myStaticLastInd.GetCriterion( 0 );
		for (int i = 0; i < nparam; i++) {
			double min=probl->GetFuncLower(i);
			double max=probl->GetFuncUpper(i);
			fval*=(myStaticLastInd.GetCriterion( i )-min)/(max-min);
		}
*/
	}
}

void objMO(int nparam, int j, double *x, double *fj, void *cd) {
	j--;//1-based
	ensureEvaluated(cd, x, nparam);
	*fj=/*fval;*/myStaticLastInd.GetCriterion( j );
}

void cntr32(int nparam, int j, double *x, double *gj, void *cd) {
	j--;//1-based
	ensureEvaluated(cd, x, nparam);
	double diff=myStaticLastInd.GetCriterion( j )-lastEps[j];
	*gj=diff>0?diff:0;
/*
switch (j) {
      case 1:
	 *gj=pow(x[0],3.e0)-6.e0*x[1]-4.e0*x[2]+3.e0;
	 break;
      case 2:
	 *gj=1.e0-x[0]-x[1]-x[2];
	 break;
   }
   return;*/
}

#include "cfsqp.h"
void DemowsaLocalsearch::improve(DemoInd &ind, Probl &probl, int miter) {
	//PREPARE INPUT PARAMETERS FOR LOCAL SEARCH
	int nparam = probl.GetNumVar();//D
	int nf = probl.GetNumFunc();   //M
	switch (nf) {
		case 2:case 3:
			if (miter>50) miter=50;//LS_FES
		default://5
			if (miter>50) miter=50;//LS_FES
	}

	double x[nparam], bl[nparam], bu[nparam];
	double bigbnd=0;//role of infinity, must be greater than each bl[i], bu[i]
	for (int i = 0; i < nparam; i++) {
		x[i] = ind.GetGene(i);				//x_i
		bl[i] = probl.GetVarLower(i);		//x_i_min
		bu[i] = probl.GetVarUpper(i);		//x_i_max
		if (-bl[i]>bigbnd) bigbnd=-bl[i]*1.1;
		if (bu[i]>bigbnd) bigbnd=bu[i]*1.1;
	}


	int nineqn=0;//nr. nonlinear in-equality constraints
	int nineq =nf;//nr. (any)     in-equality constraints

	int neqn=0; //nr. nonlinear equality constraints
	int neq =0; //nr. (any)	    equality constraints


	int nfsr=0;//number of SETS of sequentially related objective functions
	int ncsrl=0;//number of SETS of sequentially related linear constraints
	int ncsrn=0;//number of SETS of sequentially related non-linear constraints
	int mesh_pts[1]={0};//array of size of SETS of sequentially related funcs&constr, at least 1

	double eps=1.e-9;//solution distance from optimum
	double epsneq=0.e0;//allowed max. constraints violation


	double udelta=0.e0;//size of gradient approximation by final difference
	int mode=100;//100=constraints checking (1), algorithm type (FSQP-AL), problem type (P)

	int iprint=0;//desired output type (print); 1 = print objective+constraint values

	void *cd=&probl;  //client-data pointer, passed to the given objective function
	int inform;//exit status of cfsqp
	double f[nf],g[nineq+neq],lambda[nineq+neq+nf+nparam];//output data of the algorithm

	firstCallToEval=true;//for constraint's epsilon setting

	//CALL C-FSQP
	cfsqp(nparam,nf,nfsr,nineqn,nineq,neqn,neq,ncsrl,ncsrn,mesh_pts,//10
         mode,iprint,miter,&inform,bigbnd,eps,epsneq,udelta,bl,bu,x,//11
         f,g,lambda,
		 objMO,cntr32/*0*/,/*grob32*/grobfd,/*grcn32*/grcnfd,
   		 cd);//8


	//SAVE OUTPUT FROM LOCAL SEARCH
	for (int i = 0; i < nparam; i++)
		ind.SetGene(i, x[i]);	//save x_i
	for (int i = 0; i < nf; i++)
		ind.SetCriterion(i, f[i]);
}

/*
void obj32(int nparam, int j, double *x, double *fj, void *cd);
void cntr32(int nparam, int j, double *x, double *gj, void *cd);
void grob32(int nparam, int j, double *x, double *gradfj,
			 void (* dummy)(int, int, double*, double*, void*), void *cd);
void grcn32(int nparam, int j, double *x, double *gradgj,
			 void (* dummy)(int, int, double*, double*, void*), void *cd);

void obj32(int nparam, int j, double *x, double *fj, void *cd) {
   *fj=pow((x[0]+3.e0*x[1]+x[2]),2.e0)+4.e0*pow((x[0]-x[1]),2.e0);
   return;
}

void cntr32(int nparam, int j, double *x, double *gj, void *cd) {
   switch (j) {
      case 1:
	 *gj=pow(x[0],3.e0)-6.e0*x[1]-4.e0*x[2]+3.e0;
	 break;
      case 2:
	 *gj=1.e0-x[0]-x[1]-x[2];
	 break;
   }
   return;
}

void    cfsqp(int,int,int,int,int,int,int,int,int,int *,//10
			  int,int,int,int *,double,double,double,double,double *, double *,double *,//11
			  double *,double *,double *,//3 ->24
              void (*)(int,int,double *,double *,void *),			//25: obj32
              void (*)(int,int,double *,double *,void *),			//26: cntr32
              void (*)(int,int,double *,double *,					//27: grob32
                   void (*)(int,int,double *,double *,void *),void *),
              void (*)(int,int,double *,double *,					//28: grcn32
                   void (*)(int,int,double *,double *,void *),void *),
              void *);												//29: cd

void grob32(int nparam, int j, double *x, double *gradfj, void (* dummy)(int, int, double*, double*, void*), void *cd) {
   double fa,fb;

   fa=2.e0*(x[0]+3.e0*x[1]+x[2]);
   fb=8.e0*(x[0]-x[1]);
   gradfj[0]=fa+fb;
   gradfj[1]=fa*3.e0-fb;
   gradfj[2]=fa;
   return;
}

void grcn32(int nparam, int j, double *x, double *gradgj, void (* dummy)(int, int, double*, double*, void*), void *cd) {
   switch (j) {
      case 1:
	 gradgj[0]=3.e0*x[0]*x[0];
	 gradgj[1]=-6.e0;
	 gradgj[2]=-4.e0;
	 break;
      case 2:
	 gradgj[0]=gradgj[1]=gradgj[2]=-1.e0;
	 break;
   }
   return;
}
*/
