/*===========================================================================

	Copyright  2007 Tea Tuar

	This material is provided "as is", with no warranty expressed or implied.
	Any use is at your own risk. Permission to use or copy this software for
	any purpose is hereby granted without fee, provided this notice is
	retained on all copies. Permission to modify the code and to distribute
	modified code is granted, provided a notice that the code was modified is
	included with the above copyright notice.

	http://dis.ijs.si/tea/research.htm

	===========================================================================

	Implementation file.

	file: file_stat.cpp
	author: Tea Tusar (tea.tusar@ijs.si)
	created: 20.02.2006
	last change: $date$

	===========================================================================
*/

#include "file_stat.h"
//---------------------------------------------------------------------------

FileStat::FileStat()
{
	m_gen = new File();
	m_front = new File();
	m_log = new File();
	m_lines = new File();

	m_front_gen = 1;
	m_front_long = false;

	m_repl_first = 0;
	m_repl_second = 0;
	m_repl_none = 0;
}
//---------------------------------------------------------------------------

FileStat::~FileStat()
{
	delete m_gen;
	delete m_front;
	delete m_log;
	delete m_lines;
}
//---------------------------------------------------------------------------

void FileStat::InitFileStat()
{
	m_gen->InitFile();
	m_front->InitFile();
	m_log->InitFile();
	m_lines->InitFile();
}
//---------------------------------------------------------------------------

void FileStat::OutputInd(DemoInd *ind,
												 DemoInd *parent,
												 UINT index,
												 UINT domination)
{
	if (!m_log->IsOff()) {
		stringstream tmp1;
		tmp1 << index << "\t" << ind->GetIndString();
		m_log->Output(tmp1.str());
	}

	if ((domination == g_domin_first) && (!m_lines->IsOff())) {
		stringstream tmp2;
		tmp2 << index << "\t" << ind->GetIndCritString();
		tmp2 << index << "\t" << parent->GetIndCritString();
		tmp2 << endl;
		m_lines->Output(tmp2.str());
	}

	if (!m_gen->IsOff()) {
		if (domination == g_domin_first)
			m_repl_first++;
		else if (domination == g_domin_second)
			m_repl_second++;
		else
			m_repl_none++;
	}
}
//---------------------------------------------------------------------------

void FileStat::OutputPop(DemoPop *pop, UINT gen, vector<DemoInd*> *m_appSet_copy)
{
	UINT evals = pop->GetProbl()->GetNumEval();
	UINT evalsPrev = evals - pop->GetInds().size();
	if (/*( (evals >= 5000 && evalsPrev<5000)
		|| (evals >= 50000 && evalsPrev<50000)
		|| (evals >= 500000 && evalsPrev<500000)
		) &&*/ (!m_front->IsOff())) {

		if (m_params->strategyPrunning==4) {
			///SET NP
			int NPold=m_params->GetPopSize();
			int NPnew=m_params->GetPopSize()/m_params->NP_size_factor /* m_params->approximationSetSize*/;
			vector<DemoInd*> m_indsOut;
			bool reduced=false;

			///only do reduction, if needed
			if ((int)m_appSet_copy->size()>NPnew) {///if arch too big... reduce

				cout << "ASZ=" <<(int)m_appSet_copy->size()<<endl;
				cout << "NPnew="<<NPnew <<endl;
				//NPnew=(int)m_appSet_copy.size();
				///save old Individuals, send NP
				m_params->SetPopSize(NPnew);
				const vector<DemoInd*> &m_indsIn=*m_appSet_copy;
				
				///do reduction: swap indices
				pop->swapInds(m_indsIn, m_indsOut);
				vector<UINT> indexes = pop->SortPop();
				///WARNING: this reduces the size of archive,
				///by deleting several individuals, so a copy (m_appSet_copy)
				///is to be sent to this routine!!!
				pop->TruncPop(indexes);
				reduced=true;
			}

			///do print-out
			stringstream tmp1;
			tmp1 << pop->GetFrontString(m_front_long) << endl;
			m_front->Output(tmp1.str());

			if (reduced) {
				///restore indices and the NP
				//pop->returnInds(m_indsOut);
				vector<DemoInd*> m_indsOutIntermediateForWipeout;
				pop->swapInds(m_indsOut, m_indsOutIntermediateForWipeout);///get all rest from copy out for wipeout
				
				/// WIPEOUT start
				for (int i = 0; i < (int)m_indsOutIntermediateForWipeout.size(); i++)
					delete m_indsOutIntermediateForWipeout[i];
				m_indsOutIntermediateForWipeout.clear();
				/// WIPEOUT end
				
				m_params->SetPopSize(NPold);
			}
		} else {
			stringstream tmp1;
			tmp1 << pop->GetFrontString(m_front_long) << endl;
			m_front->Output(tmp1.str());
		}
	}

	if (!m_gen->IsOff()) {
		UINT num_feasible = 0;
		UINT num_unfeasible = 0;
		UINT num_non_evaluated = 0;
		double avg_violation = 0;
		for (UINT i = 0; i < pop->GetInds().size(); i++) {
			if (pop->GetInd(i)->GetEvaluated() == false) {
				num_non_evaluated++;
			}
			else {
				if (pop->GetInd(i)->GetViolation() == 0) {
					num_feasible++;
				}
				else {
					avg_violation += pop->GetInd(i)->GetViolation();
					num_unfeasible++;
				}
			}
		}
		if (num_unfeasible != 0) {
			avg_violation = avg_violation / num_unfeasible;
		}

		stringstream tmp2;
		if (gen == 1) tmp2 << endl;
		tmp2.precision(g_out_prec);
		tmp2 << scientific << gen << "\t";
		tmp2 << scientific << num_feasible << "\t";
		tmp2 << scientific << num_unfeasible << "\t";
		tmp2 << scientific << num_non_evaluated << "\t";
		tmp2 << scientific << avg_violation << "\t";

		HsoPop *hso_pop = new HsoPop(pop);
		tmp2 << scientific << hso_pop->GetHypMeasure() << "\t";
		tmp2 << scientific << hso_pop->GetIndsSize() << "\t";
		delete hso_pop;

		tmp2 << scientific << m_repl_first << "\t";
		tmp2 << scientific << m_repl_second << "\t";
		tmp2 << scientific << m_repl_none << endl;
		m_gen->Output(tmp2.str());

		m_repl_first = 0;
		m_repl_second = 0;
		m_repl_none = 0;
	}
}
//---------------------------------------------------------------------------

