/*===========================================================================

	Copyright  2007 Tea Tuar

	This material is provided "as is", with no warranty expressed or implied.
	Any use is at your own risk. Permission to use or copy this software for
	any purpose is hereby granted without fee, provided this notice is
	retained on all copies. Permission to modify the code and to distribute
	modified code is granted, provided a notice that the code was modified is
	included with the above copyright notice.

	http://dis.ijs.si/tea/research.htm

	===========================================================================

	Header file.

	Defines the class FileStat.

	file: file_stat.h
	author: Tea Tusar (tea.tusar@ijs.si)
	created: 20.02.2006
	last change: $date$

	===========================================================================
*/

#ifndef file_statH
#define file_statH
//---------------------------------------------------------------------------

#include "file.h"
#include "demo_params.h"
#include "demo_ind.h"
#include "demo_pop.h"
#include "hso_pop.h"
//---------------------------------------------------------------------------

class DemoPop;
//---------------------------------------------------------------------------

class FileStat
{
	/** File for generational statistics.
	The output format for this file is the following:
		gen_num, num_feasible, num_unfeasible, num_non_evaluated, avg_violation,
		hyp_feasible, hyp_evaluated, m_repl_first, m_repl_second, m_repl_none
	*/
	File *m_gen;

	/** File for output of fronts.
	The short output format for this file is the following:
		criteria
	The long output format for this file is the following:
		genes, is_evaluated, violation, criteria, features
	*/
	File *m_front;

	/** File for log output.
	The output format for this file is the following:
		ind_num, genes, is_evaluated, violation, criteria, features
	*/
	File *m_log;

	/** File for output of lines of improvement (lines connecting the parent and
	the candidate, which dominates the parent.
	The output format for this file is the following:
		canidate_critera, parent_criteria
	*/
	File *m_lines;

	/** Number stating which fronts are output (every m_front_gen front is
	output). */
	UINT m_front_gen;

	/** Whether to output the front with all information (long output) or only
	the criteria (short output used mainly for the test functions). */
	bool m_front_long;

	/** Number of candidates that dominate their parents in the current
	generation. */
	UINT m_repl_first;

	/** Number of candidates that are dominated by their parents in the current
	generation. */
	UINT m_repl_second;

	/** Number of candidates that are incomparable to their parents in the current
	generation. */
	UINT m_repl_none;


public:
	/** Parameters. */
	DemoParams *m_params;

	inline void SetFrontGen(UINT front_gen) {m_front_gen = front_gen;};
	inline void SetFrontLong(bool front_long) {m_front_long = front_long;};

	inline File* GetFileGen() {return m_gen;};
	inline File* GetFileFront() {return m_front;};
	inline File* GetFileLog() {return m_log;};
	inline File* GetFileLines() {return m_lines;};

	inline UINT GetFrontGen() {return m_front_gen;};
	inline bool GetFrontLong() {return m_front_long;};

	/** Class constructor. */
	FileStat();

	/** Class destructor. */
	~FileStat();

	/** Initialization function initializes all files. */
	void InitFileStat();

	/** Function outputs the information of the individual ind to files m_log
	and m_lines. It also updates the replacement statistics.
	params: ind current candidate individual
	params: parent parent of the candidate
	params: index index of the current individual
	params: domination type of domination between the individual and its parent
	 */
	void OutputInd(DemoInd *ind,
								 DemoInd *parent,
								 UINT index,
								 UINT domination);

	/** Function outputs the information regarding the current generation to
	files m_front and m_gen. It also resets the replacement statistics.
	params: pop current population
	params: gen generation number
	*/
	void OutputPop(DemoPop *pop, UINT gen, vector<DemoInd*> *m_appSet_copy);
};
//---------------------------------------------------------------------------
#endif


