/*===========================================================================

	Copyright  2007 Tea Tuar

	This material is provided "as is", with no warranty expressed or implied.
	Any use is at your own risk. Permission to use or copy this software for
	any purpose is hereby granted without fee, provided this notice is
	retained on all copies. Permission to modify the code and to distribute
	modified code is granted, provided a notice that the code was modified is
	included with the above copyright notice.

	http://dis.ijs.si/tea/research.htm

	===========================================================================

	Implementation file.

	file: hso_pop.cpp
	author: Tea Tusar (tea.tusar@ijs.si)
	created: 17.04.2006
	last change: $date$

	===========================================================================
*/

#include "hso_pop.h"
//---------------------------------------------------------------------------

HsoSlice::HsoSlice(double depth, list<DemoInd*> list)
{
	m_depth = depth;
	m_list = list;
}
//---------------------------------------------------------------------------

HsoSlice::HsoSlice(double depth, vector<DemoInd*> vector)
{
	m_depth = depth;

	m_list.clear();
	for (UINT i = 0; i < vector.size(); i++) {
		m_list.push_back(vector.at(i));
	}
}
//---------------------------------------------------------------------------

HsoSlice& HsoSlice::operator=(const HsoSlice& slice)
{
	if (this != &slice) {
		m_depth = slice.m_depth;
		m_list = slice.m_list;
	}
	return *this;
}
//---------------------------------------------------------------------------

HsoPop::HsoPop(DemoPop *pop)
{
	UINT i, j, dominate;

	/*vector<double> tmp;

	tmp.clear();
	DemoInd *ind1 = new DemoInd();
	tmp.push_back(0);
	tmp.push_back(2);
	tmp.push_back(6);
	ind1->SetCriteria(tmp);
	tmp.clear();
	DemoInd *ind2 = new DemoInd();
	tmp.push_back(2);
	tmp.push_back(4);
	tmp.push_back(5);
	ind2->SetCriteria(tmp);
	tmp.clear();
	DemoInd *ind3 = new DemoInd();
	tmp.push_back(6);
	tmp.push_back(0);
	tmp.push_back(3);
	ind3->SetCriteria(tmp);
	tmp.clear();
	DemoInd *ind4 = new DemoInd();
	tmp.push_back(8);
	tmp.push_back(3);
	tmp.push_back(0);
	ind4->SetCriteria(tmp);

	m_ref_point.push_back(11);
	m_ref_point.push_back(6);
	m_ref_point.push_back(10);

	m_ref_point.push_back(2);
	m_ref_point.push_back(2);
	m_ref_point.push_back(2);

	m_probl = pop->GetProbl();
	m_probl->SetNumFunc(3);

	m_inds.push_back(ind1);
	m_inds.push_back(ind2);
	m_inds.push_back(ind3);
	m_inds.push_back(ind4); */

	vector<UINT> n;
	n.clear();

	m_probl = pop->GetProbl();

	for (i = 0; i < pop->GetInds().size(); i++) {
		n.push_back(0);
		for (j = 0; j < pop->GetInds().size(); j++) {
			if (i == j) continue;
			dominate = m_probl->GetDomination(pop->GetInd(i), pop->GetInd(j));
			if (dominate == g_domin_second)
				n.at(i)++;
		}
		if ((n.at(i) == 0) && (pop->GetInd(i)->GetViolation() == 0) &&
			((pop->GetInd(i)->GetEvaluated() == true))) {
			DemoInd *ind = new DemoInd();
			*ind = *pop->GetInd(i);
			m_inds.push_back(ind);
		}
	}

	for (UINT j = 0; j < m_probl->GetNumFunc(); j++)
		m_ref_point.push_back(2);
}
//---------------------------------------------------------------------------

HsoPop::~HsoPop()
{
	for (UINT i = 0; i < m_inds.size(); i++) {
		delete m_inds.at(i);
	}
}
//---------------------------------------------------------------------------

double HsoPop::GetHypMeasure()
{
	if (m_inds.size() == 0) {
		return 0;
	}

	ScaleCriteria();
	SortObjective(0);

	UINT i, j, k;
	UINT n = m_probl->GetNumFunc();
	vector<HsoSlice> slices;
	vector<HsoSlice> new_slices;
	vector<HsoSlice> next_slices;
	slices.push_back(HsoSlice(1, m_inds));

	for (j = 0; j < n - 1; j++) {
		new_slices.clear();
		for (i = 0; i < slices.size(); i++) {
			next_slices = GetSlices(slices.at(i).m_list, j);
			for (k = 0; k < next_slices.size(); k++) {
				new_slices.push_back(HsoSlice(slices.at(i).m_depth *
					next_slices.at(k).m_depth,
					next_slices.at(k).m_list));
			}
		}
		slices = new_slices;
	}

	double volume = 0;
	for (i = 0; i < slices.size(); i++) {
		volume += slices.at(i).m_depth *
			fabs(slices.at(i).m_list.front()->GetCriterion(n - 1) -
			m_ref_point.at(n - 1));
	}
	return volume;
}
//---------------------------------------------------------------------------

void HsoPop::ScaleCriteria()
{
	double tmp;

	for (UINT i = 0; i < m_inds.size(); i++) {
		for (UINT j = 0; j < m_probl->GetNumFunc(); j++) {

			if (m_inds.at(i)->GetCriterion(j) < m_probl->GetFuncLower(j)) {
				m_inds.at(i)->SetCriterion(j, m_probl->GetFuncLower(j));
			}
			else if (m_inds.at(i)->GetCriterion(j) > m_probl->GetFuncUpper(j)) {
				m_inds.at(i)->SetCriterion(j, m_probl->GetFuncUpper(j));
			}

			tmp = (m_inds.at(i)->GetCriterion(j) - m_probl->GetFuncLower(j)) /
				(m_probl->GetFuncUpper(j) - m_probl->GetFuncLower(j));

			m_inds.at(i)->SetCriterion(j, tmp);
		}
	}
}
//---------------------------------------------------------------------------

void HsoPop::SortObjective(UINT index)
{
	vector<UINT> sort_index;
	vector<double> sort_value;
	vector<DemoInd*> sort_inds;

	for (UINT i = 0; i < m_inds.size(); i++)
		sort_value.push_back(m_inds.at(i)->GetCriterion(index));
	sort_index = GetSorted(sort_value);

	for (UINT i = 0; i < m_inds.size(); i++)
		sort_inds.push_back(m_inds.at(sort_index.at(i)));

	m_inds = sort_inds;
}
//---------------------------------------------------------------------------

vector<HsoSlice> HsoPop::GetSlices(list<DemoInd*> inds, UINT index)
{
	vector<HsoSlice> result;
	list<DemoInd*> new_inds;
	DemoInd *point;
	DemoInd *new_point;

	point = inds.front();
	inds.pop_front();
	new_inds.clear();
	result.clear();

	while (inds.size() > 0) {
		new_inds = Insert(point, index + 1, new_inds);
		new_point = inds.front();
		inds.pop_front();
		result.push_back(HsoSlice(
			fabs(point->GetCriterion(index) - new_point->GetCriterion(index)),
			new_inds));
		point = new_point;
	}
	new_inds = Insert(point, index + 1, new_inds);
	result.push_back(HsoSlice(
		fabs(point->GetCriterion(index) - m_ref_point.at(index)),
		new_inds));

	return result;
}
//---------------------------------------------------------------------------

list<DemoInd*> HsoPop::Insert(DemoInd *point, UINT depth, list<DemoInd*> inds)
{
	list<DemoInd*> result;
	result.clear();

	while ((inds.size() > 0) &&
		(inds.front()->GetCriterion(depth) < point->GetCriterion(depth))) {
			result.push_back(inds.front());
			inds.pop_front();
	}
	result.push_back(point);
	while (inds.size() > 0) {
		if (m_probl->GetDominationK(point, inds.front(), depth) != g_domin_first) {
			result.push_back(inds.front());
		}
		inds.pop_front();
	}

	return result;
}
//---------------------------------------------------------------------------


