/*===========================================================================

	Copyright  2007 Tea Tuar

	This material is provided "as is", with no warranty expressed or implied.
	Any use is at your own risk. Permission to use or copy this software for
	any purpose is hereby granted without fee, provided this notice is
	retained on all copies. Permission to modify the code and to distribute
	modified code is granted, provided a notice that the code was modified is
	included with the above copyright notice.

	http://dis.ijs.si/tea/research.htm

	===========================================================================

	Header file.

	Defines the struct HsoSlice and class HsoPop.

	file: hso_pop.h
	author: Tea Tusar (tea.tusar@ijs.si)
	created: 17.04.2006
	last change: $date$

	===========================================================================
*/

#ifndef hso_popH
#define hso_popH
//---------------------------------------------------------------------------

#include "demo_ind.h"
#include "demo_pop.h"
//---------------------------------------------------------------------------

class DemoPop;
//---------------------------------------------------------------------------

struct HsoSlice//Hypervolume by Slicing Objectives
{
	/** 1-dimensional volume of the current slice. */
	double m_depth;

	/** List of points (individuals) in the current slice. */
	list<DemoInd*> m_list;

	/** Structure constructor. */
	HsoSlice(double depth, list<DemoInd*> list);

	/** Structure constructor. */
	HsoSlice(double depth, vector<DemoInd*> vector);

	/** Copy operator. */
	HsoSlice& operator=(const HsoSlice& slice);
};
//---------------------------------------------------------------------------

class HsoPop
{
protected:
	/** Vector of individuals. */
	vector<DemoInd*> m_inds;

	/** Reference point. */
	vector<double> m_ref_point;

	/** Problem. */
	Probl *m_probl;

public:
	inline UINT GetIndsSize() {return m_inds.size();};

	/** Class constructor. Copies the nondominated individuals from pop to m_inds.
	params: pop pointer to the population for which the hypervolume needs to be
	calculated
	*/
	HsoPop(DemoPop *pop);

	/** Class destructor. */
	~HsoPop();

	/** Function calculates and returns the hypervolume of the current individuals.
	returns: the hypervolume of the current individuals
	*/
	double GetHypMeasure();

	/** Function scales each criterion from [fun_lower, fun_upper] to the interval
	[0, 1]. If the criteria is outside the borders, it is first scaled to fit in
	the	borders.
	*/
	void ScaleCriteria();

	/** Function sorts the individuals in ascending order according to the
	criterion given by index.
	params: index the criterion to be sorted
	*/
	void SortObjective(UINT index);

	/** Function creates and returns slices in objectives index+1-> from the list
	of individuals.
	params: inds the given individuals
	params: index the criteria index
	returns: slices in objectives index+1
	*/
	vector<HsoSlice> GetSlices(list<DemoInd*> inds, UINT index);

	/** Function accumulates incrementaly the list of individuals in each slice.
	It also eliminates the dominated individuals.
	params: point the current individual
	params: index the criteria index
	params: inds the given individuals
	returns: new list of individuals
	*/
	list<DemoInd*> Insert(DemoInd *point, UINT index, list<DemoInd*> inds);
};
//---------------------------------------------------------------------------
#endif


