/*===========================================================================

	Copyright  2007 Tea Tuar

	This material is provided "as is", with no warranty expressed or implied.
	Any use is at your own risk. Permission to use or copy this software for
	any purpose is hereby granted without fee, provided this notice is
	retained on all copies. Permission to modify the code and to distribute
	modified code is granted, provided a notice that the code was modified is
	included with the above copyright notice.

	http://dis.ijs.si/tea/research.htm

	===========================================================================

	Implementation file.

	file: probl.cpp
	author: Tea Tusar (tea.tusar@ijs.si)
	created: 18.02.2006
	last change: $date$

	===========================================================================
*/

#include "probl.h"
//---------------------------------------------------------------------------

const double g_eq_prec = 0.00000001;//1e-08
const UINT g_out_prec = 10;
//---------------------------------------------------------------------------

const UINT g_domin_none = 0;
const UINT g_domin_first = 1;
const UINT g_domin_second = 2;
const UINT g_domin_original = 3;
//---------------------------------------------------------------------------

const UINT g_prob_extern = 0;
const UINT g_prob_fun_ZDT1 = 1;
const UINT g_prob_fun_ZDT2 = 2;
const UINT g_prob_fun_ZDT3 = 3;
const UINT g_prob_fun_ZDT4 = 4;
const UINT g_prob_fun_ZDT6 = 5;
const UINT g_prob_fun_WFG1 = 6;
const UINT g_prob_fun_WFG2 = 7;
const UINT g_prob_fun_WFG3 = 8;
const UINT g_prob_fun_WFG4 = 9;
const UINT g_prob_fun_WFG5 = 10;
const UINT g_prob_fun_WFG6 = 11;
const UINT g_prob_fun_WFG7 = 12;
const UINT g_prob_fun_WFG8 = 13;
const UINT g_prob_fun_WFG9 = 14;
const UINT g_prob_fun_DTLZ1 = 15;
const UINT g_prob_fun_DTLZ2 = 16;
const UINT g_prob_fun_DTLZ3 = 17;
const UINT g_prob_fun_DTLZ4 = 18;
const UINT g_prob_fun_DTLZ5 = 19;
const UINT g_prob_fun_DTLZ6 = 20;
const UINT g_prob_fun_DTLZ7 = 21;
//CEC07
const UINT g_prob_fun_OKA2 = 101;
const UINT g_prob_fun_SYMPART = 102;
const UINT g_prob_fun_S_ZDT1 = 103;
const UINT g_prob_fun_S_ZDT2 = 104;
const UINT g_prob_fun_S_ZDT4 = 105;
const UINT g_prob_fun_R_ZDT4 = 106;
const UINT g_prob_fun_S_ZDT6 = 107;
const UINT g_prob_fun_S_DTLZ2_M3 = 108;
const UINT g_prob_fun_S_DTLZ2_M5 = 109;
const UINT g_prob_fun_R_DTLZ2_M3 = 110;
const UINT g_prob_fun_R_DTLZ2_M5 = 111;
const UINT g_prob_fun_S_DTLZ3_M3 = 112;
const UINT g_prob_fun_S_DTLZ3_M5 = 113;
const UINT g_prob_fun_WFG1_M3 = 114;
const UINT g_prob_fun_WFG1_M5 = 115;
const UINT g_prob_fun_WFG8_M3 = 116;
const UINT g_prob_fun_WFG8_M5 = 117;
const UINT g_prob_fun_WFG9_M3 = 118;
const UINT g_prob_fun_WFG9_M5 = 119;

//CEC09
const UINT g_prob_fun_UF1 = 201;
const UINT g_prob_fun_UF2 = 202;
const UINT g_prob_fun_UF3 = 203;
const UINT g_prob_fun_UF4 = 204;
const UINT g_prob_fun_UF5 = 205;
const UINT g_prob_fun_UF6 = 206;
const UINT g_prob_fun_UF7 = 207;
const UINT g_prob_fun_UF8 = 208;
const UINT g_prob_fun_UF9 = 209;
const UINT g_prob_fun_UF10 = 210;
const UINT g_prob_fun_CF1 = 211;
const UINT g_prob_fun_CF2 = 212;
const UINT g_prob_fun_CF3 = 213;
const UINT g_prob_fun_CF4 = 214;
const UINT g_prob_fun_CF5 = 215;
const UINT g_prob_fun_CF6 = 216;
const UINT g_prob_fun_CF7 = 217;
const UINT g_prob_fun_CF8 = 218;
const UINT g_prob_fun_CF9 = 219;
const UINT g_prob_fun_CF10 = 220;
const UINT g_prob_fun_R2_DTLZ2_M5 = 221;
const UINT g_prob_fun_R2_DTLZ3_M5 = 222;
const UINT g_prob_fun_D30_WFG1_M5 = 223;

const UINT g_prob_min = 0;
const UINT g_prob_max = 223;
//---------------------------------------------------------------------------

void Probl::SetVarMin(vector<double> var_min)
	{m_var_min = var_min;}

void Probl::SetVarMax(vector<double> var_max)
	{m_var_max = var_max;}

void Probl::SetVarDisc(vector<double> var_disc)
	{m_var_disc = var_disc;}

void Probl::SetFuncLower(vector<double> func_lower)
	{m_func_lower = func_lower;}

void Probl::SetFuncUpper(vector<double> func_upper)
	{m_func_upper = func_upper;}
//---------------------------------------------------------------------------

Probl::Probl()
{
	m_num_eval = 0;
	m_function = g_prob_fun_ZDT1;
	m_num_var = 30;
	m_num_func = 2;
	m_num_feat = 0;

	m_var_min.clear();
	m_var_max.clear();
	m_var_disc.clear();

	m_func_lower.clear();
	m_func_upper.clear();
}
//---------------------------------------------------------------------------

vector<double> Probl::GetRandVector()
{
	vector<double> result;
	result.clear();

	for (UINT i = 0; i < m_num_var; i++) {
		if (m_var_disc.at(i) == 0)
			result.push_back(m_var_min.at(i) + (m_var_max.at(i) - m_var_min.at(i)) *
				(double) rand() / RAND_MAX);
		else
			result.push_back(m_var_min.at(i) + m_var_disc.at(i) *
				random((UINT) (1 + (m_var_max.at(i) - m_var_min.at(i)) / m_var_disc.at(i))));
	}
	return result;
}
//---------------------------------------------------------------------------

vector<double> Probl::GetRoundVector(vector<double> &genes)
{
	vector<double> result;
	result.clear();

	for (UINT i = 0; i < m_num_var; i++) {
		if (genes.at(i) >= m_var_max.at(i))
			result.push_back(m_var_max.at(i));

		else if (genes.at(i) <= m_var_min.at(i))
			result.push_back(m_var_min.at(i));

		else if (m_var_disc.at(i) > 0) {
			long int tmp;
			if (genes.at(i) > 0) {
				tmp = (int)floor(genes.at(i) / m_var_disc.at(i) + 0.5);
			}
			else {
				tmp = (int)floor(genes.at(i) / m_var_disc.at(i) - 0.5);
			}
			result.push_back(tmp * m_var_disc.at(i));
		}
		else
			result.push_back(genes.at(i));
	}
	return result;
}
//---------------------------------------------------------------------------

bool Probl::IsEqual(vector<double> &genes1,
										vector<double> &genes2)
{
	if (genes1.size() != genes2.size())
		return false;

	for (UINT i = 0; i < genes1.size(); i++) {
		if ((m_var_disc.at(i) == 0) && (fabs(genes1.at(i) - genes2.at(i)) > g_eq_prec))
			return false;
		else if (m_var_disc.at(i) > 0 &&
			(fabs(genes1.at(i) - genes2.at(i)) > m_var_disc.at(i))) {
			return false;
		}
	}

	return true;
}
//---------------------------------------------------------------------------

UINT Probl::GetDomination(DemoInd *first, DemoInd *second)
{
// 	if (first->GetViolation() > second->GetViolation()) {
// 		return g_domin_second;
// 	}
// 	else if (first->GetViolation() < second->GetViolation()) {
// 		return g_domin_first;
// 	}
// 	else if ((first->GetViolation() == 0) && (second->GetViolation() == 0)) {



///////////// constraint handling --- by AlesZ
	if (m_num_constraints!=0) {
		double viol1=first->GetViolation();
		double viol2=second->GetViolation();
		if (epsilonConstraint>1e-6 && viol1 > epsilonConstraint && viol2 > epsilonConstraint) {
			if (viol1 > viol2)
				return g_domin_second;
			else if (viol1 < viol2)
				return g_domin_first;
		}//else compare the function values

	}
/////////////////////////


		bool first_worse = false;
		bool second_worse = false;
		double *critF=&first->GetCriteria()[0];
		double *critS=&second->GetCriteria()[0];
		double c1, c2;

		for (UINT i = 0; i < m_num_func; i++) {
			c1=critF[i];
			c2=critS[i];
			if (c1 < c2)
				second_worse = true;
			else if (c1 > c2)
				first_worse = true;
			if (second_worse && first_worse) return g_domin_none;
		}
		if (!first_worse && second_worse)
			return g_domin_first;
		else if (first_worse && !second_worse)
			return g_domin_second;
// 	}
 	return g_domin_none;
}
//---------------------------------------------------------------------------

UINT Probl::GetDominationK(DemoInd *first, DemoInd *second, UINT k)
{
	bool first_worse = false;
	bool second_worse = false;

	for (UINT i = k; i < m_num_func; i++) {
		if (first->GetCriterion(i) < second->GetCriterion(i))
			second_worse = true;
		else if (first->GetCriterion(i) > second->GetCriterion(i))
			first_worse = true;
	}
	if (!first_worse && second_worse)
		return g_domin_first;
	else if (first_worse && !second_worse)
		return g_domin_second;

	return g_domin_none;
}
//---------------------------------------------------------------------------

double Probl::GetDistance(vector<double> &crit1,
													vector<double> &crit2)
{
	double result = 0;

	if (IsEqual(crit1, crit2))
		return result;

	for (UINT i = 0; i < m_num_func; i++)
		result += pow(crit1.at(i) - crit2.at(i), 2);

	return sqrt(result);
}
//---------------------------------------------------------------------------

string Probl::GetVariableString()
{
	stringstream result;

	result.precision(g_out_prec);
	for (UINT i = 0; i < m_num_var; i++) {
		result << "\t" << scientific << m_var_min.at(i);
		result << "\t" << scientific << m_var_max.at(i);
		result << "\t" << scientific << m_var_disc.at(i) << endl;
	}

	return (result.str());
}
//---------------------------------------------------------------------------

string Probl::GetFunctionString()
{
	stringstream result;

	result.precision(g_out_prec);
	for (UINT i = 0; i < m_num_func; i++) {
		result << "\t" << scientific << m_func_lower.at(i);
		result << "\t" << scientific << m_func_upper.at(i) << endl;
	}

	return (result.str());
}
//---------------------------------------------------------------------------

string Probl::GetProblemString()
{
	string result;

	switch (m_function) {
		case g_prob_extern: result = "extern evaluation"; break;
		case g_prob_fun_ZDT1: result = "ZDT1"; break;
		case g_prob_fun_ZDT2: result = "ZDT2"; break;
		case g_prob_fun_ZDT3: result = "ZDT3"; break;
		case g_prob_fun_ZDT4: result = "ZDT4"; break;
		case g_prob_fun_ZDT6: result = "ZDT6"; break;
		case g_prob_fun_WFG1: result = "WFG1"; break;
		case g_prob_fun_WFG2: result = "WFG2"; break;
		case g_prob_fun_WFG3: result = "WFG3"; break;
		case g_prob_fun_WFG4: result = "WFG4"; break;
		case g_prob_fun_WFG5: result = "WFG5"; break;
		case g_prob_fun_WFG6: result = "WFG6"; break;
		case g_prob_fun_WFG7: result = "WFG7"; break;
		case g_prob_fun_WFG8: result = "WFG8"; break;
		case g_prob_fun_WFG9: result = "WFG9"; break;
		case g_prob_fun_DTLZ1: result = "DTLZ1"; break;
		case g_prob_fun_DTLZ2: result = "DTLZ2"; break;
		case g_prob_fun_DTLZ3: result = "DTLZ3"; break;
		case g_prob_fun_DTLZ4: result = "DTLZ4"; break;
		case g_prob_fun_DTLZ5: result = "DTLZ5"; break;
		case g_prob_fun_DTLZ6: result = "DTLZ6"; break;
		case g_prob_fun_DTLZ7: result = "DTLZ7"; break;
		//CEC07
		case g_prob_fun_OKA2: result = "OKA2"; break;
		case g_prob_fun_SYMPART: result = "SYMPART"; break;
		case g_prob_fun_S_ZDT1: result = "S_ZDT1"; break;
		case g_prob_fun_S_ZDT2: result = "S_ZDT2"; break;
		case g_prob_fun_S_ZDT4: result = "S_ZDT4"; break;
		case g_prob_fun_R_ZDT4: result = "R_ZDT4"; break;
		case g_prob_fun_S_ZDT6: result = "S_ZDT6"; break;
		case g_prob_fun_S_DTLZ2_M3: result = "S_DTLZ2_M3"; break;
		case g_prob_fun_S_DTLZ2_M5: result = "S_DTLZ2_M5"; break;
		case g_prob_fun_R_DTLZ2_M3: result = "R_DTLZ2_M3"; break;
		case g_prob_fun_R_DTLZ2_M5: result = "R_DTLZ2_M5"; break;
		case g_prob_fun_S_DTLZ3_M3: result = "S_DTLZ3_M3"; break;
		case g_prob_fun_S_DTLZ3_M5: result = "S_DTLZ3_M5"; break;
		case g_prob_fun_WFG1_M3: result = "WFG1_M3"; break;
		case g_prob_fun_WFG1_M5: result = "WFG1_M5"; break;
		case g_prob_fun_WFG8_M3: result = "WFG8_M3"; break;
		case g_prob_fun_WFG8_M5: result = "WFG8_M5"; break;
		case g_prob_fun_WFG9_M3: result = "WFG9_M3"; break;
		case g_prob_fun_WFG9_M5: result = "WFG9_M5"; break;
		//CEC09
		case g_prob_fun_UF1: result = "UF1"; break;
		case g_prob_fun_UF2: result = "UF2"; break;
		case g_prob_fun_UF3: result = "UF3"; break;
		case g_prob_fun_UF4: result = "UF4"; break;
		case g_prob_fun_UF5: result = "UF5"; break;
		case g_prob_fun_UF6: result = "UF6"; break;
		case g_prob_fun_UF7: result = "UF7"; break;
		case g_prob_fun_UF8: result = "UF8"; break;
		case g_prob_fun_UF9: result = "UF9"; break;
		case g_prob_fun_UF10: result = "UF10"; break;
		case g_prob_fun_CF1: result = "CF1"; break;
		case g_prob_fun_CF2: result = "CF2"; break;
		case g_prob_fun_CF3: result = "CF3"; break;
		case g_prob_fun_CF4: result = "CF4"; break;
		case g_prob_fun_CF5: result = "CF5"; break;
		case g_prob_fun_CF6: result = "CF6"; break;
		case g_prob_fun_CF7: result = "CF7"; break;
		case g_prob_fun_CF8: result = "CF8"; break;
		case g_prob_fun_CF9: result = "CF9"; break;
		case g_prob_fun_CF10: result = "CF10"; break;
		case g_prob_fun_R2_DTLZ2_M5: result = "R2_DTLZ2_M5"; break;
		case g_prob_fun_R2_DTLZ3_M5: result = "R2_DTLZ3_M5"; break;
		case g_prob_fun_D30_WFG1_M5: result = "WFG1_M5"; break;
	}
	return result;
}
//---------------------------------------------------------------------------

