/*===========================================================================

	Copyright  2007 Tea Tuar

	This material is provided "as is", with no warranty expressed or implied.
	Any use is at your own risk. Permission to use or copy this software for
	any purpose is hereby granted without fee, provided this notice is
	retained on all copies. Permission to modify the code and to distribute
	modified code is granted, provided a notice that the code was modified is
	included with the above copyright notice.

	http://dis.ijs.si/tea/research.htm

	===========================================================================

	Header file.

	Defines the class Probl.

	file: probl.h
	author: Tea Tusar (tea.tusar@ijs.si)
	created: 18.02.2006
	last change: $date$

	===========================================================================
*/

#ifndef problH
#define problH
//---------------------------------------------------------------------------

#include "demo_params.h"
#include "demo_ind.h"
//---------------------------------------------------------------------------

/** Precision for determining the equality of genes and criteria. */
extern const double g_eq_prec;

/** Precision of output. */
extern const UINT g_out_prec;
//---------------------------------------------------------------------------

/** Type of domination. */
extern const UINT g_domin_none;
extern const UINT g_domin_first;
extern const UINT g_domin_second;
extern const UINT g_domin_original;
//---------------------------------------------------------------------------

/** Type of problem. */
extern const UINT g_prob_extern;
extern const UINT g_prob_fun_ZDT1;
extern const UINT g_prob_fun_ZDT2;
extern const UINT g_prob_fun_ZDT3;
extern const UINT g_prob_fun_ZDT4;
extern const UINT g_prob_fun_ZDT6;
extern const UINT g_prob_fun_WFG1;
extern const UINT g_prob_fun_WFG2;
extern const UINT g_prob_fun_WFG3;
extern const UINT g_prob_fun_WFG4;
extern const UINT g_prob_fun_WFG5;
extern const UINT g_prob_fun_WFG6;
extern const UINT g_prob_fun_WFG7;
extern const UINT g_prob_fun_WFG8;
extern const UINT g_prob_fun_WFG9;
extern const UINT g_prob_fun_DTLZ1;
extern const UINT g_prob_fun_DTLZ2;
extern const UINT g_prob_fun_DTLZ3;
extern const UINT g_prob_fun_DTLZ4;
extern const UINT g_prob_fun_DTLZ5;
extern const UINT g_prob_fun_DTLZ6;
extern const UINT g_prob_fun_DTLZ7;
//CEC07
extern const UINT g_prob_fun_OKA2;
extern const UINT g_prob_fun_SYMPART;
extern const UINT g_prob_fun_S_ZDT1;
extern const UINT g_prob_fun_S_ZDT2;
extern const UINT g_prob_fun_S_ZDT4;
extern const UINT g_prob_fun_R_ZDT4;
extern const UINT g_prob_fun_S_ZDT6;
extern const UINT g_prob_fun_S_DTLZ2_M3;
extern const UINT g_prob_fun_S_DTLZ2_M5;
extern const UINT g_prob_fun_R_DTLZ2_M3;
extern const UINT g_prob_fun_R_DTLZ2_M5;
extern const UINT g_prob_fun_S_DTLZ3_M3;
extern const UINT g_prob_fun_S_DTLZ3_M5;
extern const UINT g_prob_fun_WFG1_M3;
extern const UINT g_prob_fun_WFG1_M5;
extern const UINT g_prob_fun_WFG8_M3;
extern const UINT g_prob_fun_WFG8_M5;
extern const UINT g_prob_fun_WFG9_M3;
extern const UINT g_prob_fun_WFG9_M5;

//CEC09
extern const UINT g_prob_fun_UF1;
extern const UINT g_prob_fun_UF2;
extern const UINT g_prob_fun_UF3;
extern const UINT g_prob_fun_UF4;
extern const UINT g_prob_fun_UF5;
extern const UINT g_prob_fun_UF6;
extern const UINT g_prob_fun_UF7;
extern const UINT g_prob_fun_UF8;
extern const UINT g_prob_fun_UF9;
extern const UINT g_prob_fun_UF10;
extern const UINT g_prob_fun_CF1;
extern const UINT g_prob_fun_CF2;
extern const UINT g_prob_fun_CF3;
extern const UINT g_prob_fun_CF4;
extern const UINT g_prob_fun_CF5;
extern const UINT g_prob_fun_CF6;
extern const UINT g_prob_fun_CF7;
extern const UINT g_prob_fun_CF8;
extern const UINT g_prob_fun_CF9;
extern const UINT g_prob_fun_CF10;
extern const UINT g_prob_fun_R2_DTLZ2_M5;
extern const UINT g_prob_fun_R2_DTLZ3_M5;
extern const UINT g_prob_fun_D30_WFG1_M5;


extern const UINT g_prob_min;
extern const UINT g_prob_max;
//---------------------------------------------------------------------------

class DemoInd;
//---------------------------------------------------------------------------

class Probl
{
protected:
	/** Current number of evaluated individuals (the individuals, for which the
	evaluation is false, also count). */
	UINT m_num_eval;

	/** Problem function. */
	UINT m_function;

	/** Number of variables. */
	UINT m_num_var;

	UINT m_num_constraints;

	/** Number of functions. */
	UINT m_num_func;

	/** Number of features. */
	UINT m_num_feat;

	/** Vector of minimum values for every variable. */
	vector<double> m_var_min;

	/** Vector of maximum values for every variable. */
	vector<double> m_var_max;

	/** Vector of variable discretization for every variable. */
	vector<double> m_var_disc;

	/** Vector of lower bounds for every function.
	Was it supposed to be used for calculating hypervolume?!? */
	vector<double> m_func_lower;

	/** Vector of upper bounds for every function. */
	vector<double> m_func_upper;


public:
	double epsilonConstraint;
	inline void SetFunction(UINT function) {m_function = function;};
	inline void IncNumEval() {m_num_eval++;};

	inline void SetNumVar(UINT num_var) {m_num_var = num_var;};
	inline void SetNumFunc(UINT num_func) {m_num_func = num_func;};
	inline void SetNumFeat(UINT num_feat) {m_num_feat = num_feat;};

	void SetVarMin(vector<double> var_min);
	void SetVarMax(vector<double> var_max);
	void SetVarDisc(vector<double> var_disc);

	void SetFuncLower(vector<double> func_lower);
	void SetFuncUpper(vector<double> func_upper);

	inline UINT GetNumEval() {return m_num_eval;};
	inline UINT GetFunction() {return m_function;};

	inline UINT GetNumVar() {return m_num_var;};
	inline UINT GetNumFunc() {return m_num_func;};
	inline UINT GetNumFeat() {return m_num_feat;};

	inline double GetFuncLower(UINT index) {return m_func_lower.at(index);};
	inline double GetFuncUpper(UINT index) {return m_func_upper.at(index);};

	inline double GetVarLower(UINT index) {return m_var_min[index];}
	inline double GetVarUpper(UINT index) {return m_var_max[index];}

	/** Class constructor. */
	Probl();
	virtual ~Probl(){};

	/** Virtual initialization function - to be overrided by the descendent
	classes. */
	virtual void InitProblem() = 0;

	/** Virtual function for evaluating the individual ind - to be overrided by
	the descendent classes.
	params: ind the individual to be evaluated
	*/
	virtual void Evaluate(DemoInd *ind) = 0;

	/** Function returns a random vector within the limits defined by m_var_min,
	m_var_max and m_var_disc.
	returns: a random vector
	*/
	vector<double> GetRandVector();

	/** Function rounds the given vector to fit into the limits defined by
	m_var_min, m_var_max and m_var_disc.
	params: genes the given vector
	returns: the rounded vector.
	*/
	vector<double> GetRoundVector(vector<double> &genes);

	/** Function establishes if the vectors are equal (with regard to the given
	precision.
	params: genes1 first vector
	params: genes2 second vector
	returns: true if the vectors are equal and false otherwises
	*/
	bool IsEqual(vector<double> &genes1,
							 vector<double> &genes2);

	/** Function establishes the type of domination between the given individuals.
	params: first the first individual
	params: second the second individual
	returns: type of domination between the individuals
	*/
	UINT GetDomination(DemoInd *first, DemoInd *second);

	/** Function establishes the type of domination regarding the objectives from
	k to num_func between the given individuals.
	params: first the first individual
	params: second the second individual
	params: k the first objective taken into account
	returns: type of domination between the individuals
	*/
	UINT GetDominationK(DemoInd *first, DemoInd *second, UINT k);

	/** Function calculates and returns the distance between two given vectors.
	params: crit1 the first vector
	params: crit2 the second vector
	returns: the distance between the given vectors
	*/
	double GetDistance(vector<double> &crit1,
										 vector<double> &crit2);

	/** Function constructs and returns the string with information on variable
	domains.
	returns: a string with information on variable domains
	*/
	string GetVariableString();

	/** Function constructs and returns the string with information on function
	bounds.
	returns: a string with information on function bounds
	*/
	string GetFunctionString();

	/** Function returns the string with problem name.
	returns: a string with the name of the given problem
	*/
	string GetProblemString();
};
//---------------------------------------------------------------------------
#endif

