/*===========================================================================

	Copyright  2007 Tea Tuar

	This material is provided "as is", with no warranty expressed or implied.
	Any use is at your own risk. Permission to use or copy this software for
	any purpose is hereby granted without fee, provided this notice is
	retained on all copies. Permission to modify the code and to distribute
	modified code is granted, provided a notice that the code was modified is
	included with the above copyright notice.

	http://dis.ijs.si/tea/research.htm

	===========================================================================

	Implementation file.

	file: probl_sim.cpp
	author: Tea Tusar (tea.tusar@ijs.si)
	created: 18.02.2006
	last change: $date$

	===========================================================================
*/

#include "probl_sim.h"
//#include <windows>
//---------------------------------------------------------------------------

void ProblSim::SetCommand(string command)
	{m_command = command;}

void ProblSim::SetArchive(string archive)
	{m_archive = archive;}

void ProblSim::SetSimIn(string sim_in)
	{m_sim_in = sim_in;}

void ProblSim::SetSimOut(string sim_out)
	{m_sim_out = sim_out;}

void ProblSim::SetArchiveRead(bool archive_read)
	{m_archive_read = archive_read;}

void ProblSim::SetArchiveWrite(bool archive_write)
	{m_archive_write = archive_write;}
//---------------------------------------------------------------------------

ProblSim::ProblSim()
	: Probl()
{
	m_command = "simulator.bat",
	m_archive = "archive.txt";
	m_sim_in = "sim_in.txt";
	m_sim_out = "sim_out.txt";

	m_archive_read = false;
	m_archive_write = false;
}
//---------------------------------------------------------------------------

ProblSim::~ProblSim()
{
}
//---------------------------------------------------------------------------

void ProblSim::InitProblem()
{
}
//---------------------------------------------------------------------------

void ProblSim::Evaluate(DemoInd *ind)
{
	cout << "\revaluating individual " << GetNumEval() << flush;
	if (!IsInArchive(ind)) {
		WriteSimIn(ind->GetGenes());
		FileExec(m_command, true, true);
		ReadSimOut(ind);
		if (m_archive_write)
			WriteToArchive(ind);
	}
}
//---------------------------------------------------------------------------

bool ProblSim::IsInArchive(DemoInd *ind)
{
	if (!m_archive_read)
		return false;

	ifstream f;
	f.open(m_archive.c_str(), fstream::in);

	try {
		if (!f.is_open())
			throw E("Could not open the file " + m_archive + "\n");
	} catch (E &e) {
		cerr << e.error << endl;
		return false;
	}

	DemoInd *archive_ind = new DemoInd();
	double tmp_d;
	bool tmp_b;
	vector<double> tmp_vect;

	while (!f.eof()) {
		tmp_vect.clear();
		for (UINT i = 0; i < m_num_var; i++) {
			f >> tmp_d;
			tmp_vect.push_back(tmp_d);
		}
		archive_ind->SetGenes(tmp_vect);
		f >> tmp_b;
		archive_ind->SetEvaluated(tmp_b);
		f >> tmp_d;
		archive_ind->SetViolation(tmp_d);

		tmp_vect.clear();
		for (UINT i = 0; i < m_num_func; i++) {
			f >> tmp_d;
			tmp_vect.push_back(tmp_d);
		}
		archive_ind->SetCriteria(tmp_vect);

		tmp_vect.clear();
		for (UINT i = 0; i < m_num_feat; i++) {
			f >> tmp_d;
			tmp_vect.push_back(tmp_d);
		}
		archive_ind->SetFeatures(tmp_vect);

		if (IsEqual(ind->GetGenes(), archive_ind->GetGenes())) {
			*ind = *archive_ind;
			f.close();
			return true;
		}
	}

	f.close();
	delete archive_ind;
	return false;
}
//---------------------------------------------------------------------------

void ProblSim::WriteSimIn(vector<double> &genes)
{
	ofstream f;
	f.open(m_sim_in.c_str(), fstream::out);

	try {
		if (!f.is_open())
			throw E("Could not open the file " + m_sim_in + "\n");
	} catch (E &e) {
		cerr << e.error << endl;
		set_terminate(my_terminate);
		terminate();
	}

	f.precision(g_out_prec);
	for (UINT i = 0; i < genes.size(); i++)
		f << scientific << genes.at(i) << endl;
	f.close();
}
//---------------------------------------------------------------------------

void ProblSim::FileExec(string command,
												bool hide,
												bool wait)
{
	std::cerr << "Executed file!!" << std::endl;
	exit(1);
	/*STARTUPINFO StartupInfo;
	PROCESS_INFORMATION ProcessInfo;

	try {
		StartupInfo.cb = sizeof(STARTUPINFO);
		StartupInfo.dwFlags = STARTF_USESHOWWINDOW || STARTF_FORCEONFEEDBACK;
		if (hide)
			StartupInfo.wShowWindow = SW_HIDE;
		else
			StartupInfo.wShowWindow = SW_SHOWNORMAL;
		StartupInfo.lpReserved = NULL;
		StartupInfo.lpDesktop = NULL;
		StartupInfo.lpTitle = NULL;
		StartupInfo.cbReserved2 = 0;
		StartupInfo.lpReserved2 = NULL;

		try {
			CreateProcess(NULL, (char*)command.c_str(), NULL, NULL, false,
				NORMAL_PRIORITY_CLASS, NULL, NULL, &StartupInfo, &ProcessInfo);
		} catch (...) {
			cerr << "Could not run command " << command << endl;
			set_terminate(my_terminate);
			terminate();
		}

		if (wait)
			do {
			}
			while (WaitForSingleObject(ProcessInfo.hProcess, 100) == WAIT_TIMEOUT);

	} __finally {
		CloseHandle(ProcessInfo.hProcess);
		CloseHandle(ProcessInfo.hThread);
	}*/
}
//---------------------------------------------------------------------------

void ProblSim::ReadSimOut(DemoInd* ind)
{
	ifstream f;
	f.open(m_sim_out.c_str(), fstream::in);

	try {
		if (!f.is_open())
			throw E("Could not open the file " + m_sim_out + "\n");
	} catch (E &e) {
		cerr << e.error << endl;
		set_terminate(my_terminate);
		terminate();
	}

	double tmp_d;
	bool tmp_b;
	vector<double> tmp_vect;

	try {

		f >> tmp_b;
		ind->SetEvaluated(tmp_b);
		f >> tmp_d;
		ind->SetViolation(tmp_d);

		tmp_vect.clear();
		for (UINT i = 0; i < m_num_func; i++) {
			f >> tmp_d;
			tmp_vect.push_back(tmp_d);
		}
		ind->SetCriteria(tmp_vect);

		tmp_vect.clear();
		for (UINT i = 0; i < m_num_feat; i++) {
			f >> tmp_d;
			tmp_vect.push_back(tmp_d);
		}
		ind->SetFeatures(tmp_vect);

	} catch (...) {
		cerr << "An error occured while reading the simulator output" << endl;
		f.close();
		set_terminate(my_terminate);
		terminate();
	}
	f.close();
}
//---------------------------------------------------------------------------

void ProblSim::WriteToArchive(DemoInd* ind)
{
	ofstream f;
	f.open(m_archive.c_str(), fstream::app);

	try {
		if (!f.is_open())
			throw E("Could not open the file " + m_archive + "\n");
	} catch (E &e) {
		cerr << e.error << endl;
		set_terminate(my_terminate);
		terminate();
	}

	UINT i;
	f.precision(g_out_prec);
	for (i = 0; i < m_num_var; i++)
		f << scientific << ind->GetGene(i) << "\t";

	f << GetBoolString01(ind->GetEvaluated()) << "\t";
	f << scientific << ind->GetViolation() << "\t";

	for (i = 0; i < m_num_func; i++)
		f << scientific << ind->GetCriterion(i) << "\t";

	for (i = 0; i < m_num_feat; i++)
		f << scientific << ind->GetFeature(i) << "\t";

	f << endl;
	f.close();
}
//---------------------------------------------------------------------------

