/*===========================================================================

	Copyright  2005-2007 Tea Tuar

	This material is provided "as is", with no warranty expressed or implied.
	Any use is at your own risk. Permission to use or copy this software for
	any purpose is hereby granted without fee, provided this notice is
	retained on all copies. Permission to modify the code and to distribute
	modified code is granted, provided a notice that the code was modified is
	included with the above copyright notice.

	http://dis.ijs.si/tea/research.htm

	===========================================================================

	Header file.

	Defines the class ProblSim, which is derived from the class Probl.

	file: probl_sim.h
	author: Tea Tusar (tea.tusar@ijs.si)
	created: 18.02.2006
	last change: $date$

	===========================================================================
*/

#ifndef probl_simH
#define probl_simH
//---------------------------------------------------------------------------

#include "demo_params.h"
#include "demo_utilities.h"
#include "probl.h"
//---------------------------------------------------------------------------

class ProblSim : public Probl
{
	/** Command to call the simulator. */
	string m_command;

	/** File name of the archive. */
	string m_archive;

	/** File name of the file with input for the simulator. */
	string m_sim_in;

	/** File name of the file with output of the simulator. */
	string m_sim_out;

	/** Whether to read individuals from the archive. */
	bool m_archive_read;

	/** Whether to write individuals to the archive. */
	bool m_archive_write;

public:
	void SetCommand(string command);
	void SetArchive(string archive);
	void SetSimIn(string sim_in);
	void SetSimOut(string sim_out);

	void SetArchiveRead(bool archive_read);
	void SetArchiveWrite(bool archive_write);

	inline string GetCommand() {return m_command;};
	inline string GetArchive() {return m_archive;};
	inline string GetSimIn() {return m_sim_in;};
	inline string GetSimOut() {return m_sim_out;};

	inline bool GetArchiveRead() {return m_archive_read;};
	inline bool GetArchiveWrite() {return m_archive_write;};

	/** Class constructor. */
	ProblSim();

	/** Class destructor. */
	virtual ~ProblSim();

	/** Initialization function (empty). */
	void InitProblem();

	/** Function calls the evaluation of the current individual. The individual
	can be read from the archive or an extern simulator is called.
	params: ind the current individual
	*/
	void Evaluate(DemoInd *ind);

	/** Function checks if the current individual has already been evaluated.
	params: ind the current individual
	returns: true if the individual exists in archive and false otherwise
	throws: exception if the archive cannot be opened for reading
	*/
	bool IsInArchive(DemoInd *ind);

	/** Function writes the parameters to the simulator input file.
	params: genes the parameters to be written
	throws: exception if the simulator input file cannot be opened for writing
	*/
	void WriteSimIn(vector<double> &genes);

	/** Function calls the process defined by command.
	params: command the name of the process
	params: hide whether to hide the process window
	params: wait whether to wait for the process to finish before continuing
	throws: exception if the process cannot be started
	*/
	void FileExec(string command,
								bool hide,
								bool wait);

	/** Function reads the simulator output and stores the criteria in the
	current individual.
	params: ind current individual
	throws: exception if the simulator output file cannot be opened for reading
	throws: exception if an error occurs while reading the simulator output
	*/
	void ReadSimOut(DemoInd* ind);

	/** Function writes the information on the individual to the archive.
	params: ind the current individual
	throws: exception if the archive cannot be opened for writing
	*/
	void WriteToArchive(DemoInd *ind);
};
//---------------------------------------------------------------------------
#endif

