/***************************************************************************
 *   Copyright (C) 2007 by Aleš Zamuda, Tea Tušar (IJS)   *
 *   ales.zamuda@uni-mb.si, tea.tusar@ijs.si   *
 *                                                                         *
 *   Permission is hereby granted, free of charge, to any person obtaining *
 *   a copy of this software and associated documentation files (the       *
 *   "Software"), to deal in the Software without restriction, including   *
 *   without limitation the rights to use, copy, modify, merge, publish,   *
 *   distribute, sublicense, and/or sell copies of the Software, and to    *
 *   permit persons to whom the Software is furnished to do so, subject to *
 *   the following conditions:                                             *
 *                                                                         *
 *   The above copyright notice and this permission notice shall be        *
 *   included in all copies or substantial portions of the Software.       *
 *                                                                         *
 *   THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,       *
 *   EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF    *
 *   MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. *
 *   IN NO EVENT SHALL THE AUTHORS BE LIABLE FOR ANY CLAIM, DAMAGES OR     *
 *   OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, *
 *   ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR *
 *   OTHER DEALINGS IN THE SOFTWARE.                                       *
 ***************************************************************************/
#ifndef PROBLFUNCEC07_H
#define PROBLFUNCEC07_H

#include "probl_fun.h"

/**
*	@author Aleš Zamuda <ales.zamuda@uni-mb.si>
 * This class contains the problem functions for CEC07 problem suite,
 * which is copyrighted by other authors too -- see technical report
 * for performance assessment of MO at CEC07.
*/
class ProblFunCec07 : public ProblFun {
public:
	ProblFunCec07();
	~ProblFunCec07();

	/** Initialization function sets the parameters of the problem. */
	void InitProblem();

	/** Function calls the evaluation of the current individual on a CEC07
	 * test function.
	params: ind the current individual
	*/
	void Evaluate(DemoInd *ind);

	void CalcOKA2(DemoInd *ind);
	void CalcSYMPART(DemoInd *ind);
	void CalcS_ZDT1(DemoInd *ind);
	void CalcS_ZDT2(DemoInd *ind);
	void CalcS_ZDT4(DemoInd *ind);
	void CalcR_ZDT4(DemoInd *ind);
	void CalcS_ZDT6(DemoInd *ind);
	void CalcS_DTLZ2_M3(DemoInd *ind);
	void CalcS_DTLZ2_M5(DemoInd *ind);
	void CalcR_DTLZ2_M3(DemoInd *ind);
	void CalcR_DTLZ2_M5(DemoInd *ind);
	void CalcS_DTLZ3_M3(DemoInd *ind);
	void CalcS_DTLZ3_M5(DemoInd *ind);
	void CalcWFG1_M3(DemoInd *ind);
	void CalcWFG1_M5(DemoInd *ind);
	void CalcWFG8_M3(DemoInd *ind);
	void CalcWFG8_M5(DemoInd *ind);
	void CalcWFG9_M3(DemoInd *ind);
	void CalcWFG9_M5(DemoInd *ind);

protected:
	void test_OKA2(const vector<double> &x, vector<double> &f,int nx,int n_obj);
	void test_SYMPART(const vector<double> &x, vector<double> &f,int nx,int n_obj);
	void test_S_ZDT1(const vector<double> &x, vector<double> &f,int nx,int n_obj);
	void test_S_ZDT2(const vector<double> &x, vector<double> &f,int nx,int n_obj);
	void test_S_ZDT4(const vector<double> &x, vector<double> &f,int nx,int n_obj);
	void test_R_ZDT4(const vector<double> &x, vector<double> &f,int nx,int n_obj);
	void test_S_ZDT6(const vector<double> &x, vector<double> &f,int nx,int n_obj);
	void test_S_DTLZ2(const vector<double> &x, vector<double> &f,int nx,int n_obj);
	void test_R_DTLZ2(const vector<double> &x, vector<double> &f,int nx,int n_obj);
	void test_S_DTLZ3(const vector<double> &x, vector<double> &f,int nx,int n_obj);
	void test_WFG1(const vector<double> &z, vector<double> &f, int nx,  int M);
	void test_WFG8(const vector<double> &x, vector<double> &f,int nx,int M);
	void test_WFG9(const vector<double> &x, vector<double> &f,int nx,int M);

private:
	//SYMPART utility functions
	static void findTile(double x1, double x2, int *t1, int *t2);
	static int findTileSYMPART(double x1, double x2);

	static double correct_to_01(double aa, double epsilon);
	static int vector_in_01( const double *x, int nx );
	static double linear( const double *x, const int m,int M );
	static double convex( const double *x, const int m,int M );
	static double concave( const double *x, const int m,int M );
	static double mixed( const double *x, const int A, const double  alpha );
	static double disc( const double *x, const int A, const double alpha, const double beta );
	static inline double min_double( const double aa, const double bb );
	static inline double b_poly( const double y, const double alpha );
	static double b_flat(const double y, const double A, const double B, const double C);
	static double b_param(const double y, const double u, const double A, const double B, const double C);
	static double s_linear( const double y, const double A );
	static double s_decept(const double y, const double A, const double B, const double C );
	static double s_multi(const double y, const int A, const double B, const double C);
	static double r_sum( const double *y, const double *w, int ny);
	static double r_nonsep( const double *y, const int A, int ny);

};

#endif
